import collections
import logging
import solv

from pulp_rpm.app import models

from django.conf import settings


logger = logging.getLogger(__name__)

# The name for the repo inside libsolv which represents the combined set of target/destination
# repositories. Libsolv only supports one "installed" repo at a time, therefore we need to
# combine them and determine what units actually go where afterwards.
COMBINED_TARGET_REPO_NAME = "combined_target_repo"

# Constants for loading data from the database.
RPM_FIELDS = [
    "pk",
    "name",
    "version",
    "release",
    "epoch",
    "arch",
    "rpm_vendor",
    "provides",
    "requires",
    "files",
]

MODULE_FIELDS = [
    "pk",
    "name",
    "stream",
    "version",
    "context",
    "arch",
    "dependencies",
    "artifacts",
]

MODULE_DEFAULTS_FIELDS = [
    "pk",
    "module",
    "stream",
    "profiles",
]


def parse_nevra(name):
    """Parse NEVRA.

    Original from Pulp 2.

    Args:
        name (str): NEVR "jay-3:3.10-4.fc3.x86_64"

    Returns: (tuple) Parsed NEVR (name: str, epoch: int, version: str, release: str, arch: str)
    """
    if name.count(".") < 1:
        raise ValueError("failed to parse nevra '%s' not a valid nevra" % name)

    arch_dot_pos = name.rfind(".")
    arch = name[arch_dot_pos + 1 :]

    return parse_nevr(name[:arch_dot_pos]) + (arch,)


def parse_nevr(name):
    """Parse NEVR.

    Originally from Pulp 2.

    Args:
        name (str): NEVR "jay-3:3.10-4.fc3"

    Returns: (tuple) Parsed NEVR (name: str, epoch: int, version: str, release: str)
    """
    if name.count("-") < 2:  # release or name is missing
        raise ValueError("failed to parse nevr '%s' not a valid nevr" % name)

    release_dash_pos = name.rfind("-")
    release = name[release_dash_pos + 1 :]
    name_epoch_version = name[:release_dash_pos]
    name_dash_pos = name_epoch_version.rfind("-")
    package_name = name_epoch_version[:name_dash_pos]

    epoch_version = name_epoch_version[name_dash_pos + 1 :].split(":")
    if len(epoch_version) == 1:
        epoch = 0
        version = epoch_version[0]
    elif len(epoch_version) == 2:
        epoch = int(epoch_version[0])
        version = epoch_version[1]
    else:
        # more than one ':'
        raise ValueError("failed to parse nevr '%s' not a valid nevr" % name)

    return package_name, epoch, version, release


def libsolv_formatted_evr(epoch, version, release):
    """Create an epoch-version-release string from the separate values.

    Pulp stores epoch-version-release separately, libsolv uses them together.
    Convert from Pulp separate values to a combined EVR formatted as libsolv expects.
    """
    # This function is sometimes used with dependencies, not just packages we know full details
    # about. So if there's no specific EVR information, we need to set EVR to None.
    if version is None:
        return None

    return "{}{}{}".format(
        "{}:".format(epoch) if epoch else "", version, "-{}".format(release) if release else ""
    )


def rpm_to_solvable(solv_repo, unit):
    """Convert a Pulp RPM dict to a libsolv solvable.

    Args:
        solv_repo (solv.Repo): The libsolv repository the unit is being created in.
        unit (dict): The unit being converted.

    Returns:
        (solv.Solvable) The solvable created.

    """
    solvable = solv_repo.add_solvable()

    def rpm_filelist_conversion(solvable, unit):
        """A specific, rpm-unit-type filelist attribute conversion."""
        repodata = solv_repo.first_repodata()

        for file_repr in unit.get("files", []):
            # file_repr = e.g. (None, '/usr/bin/', 'bash')
            file_dir = file_repr[1]
            file_name = file_repr[2]
            if not file_dir:
                # https://github.com/openSUSE/libsolv/issues/397
                continue
            dirname_id = repodata.str2dir(file_dir)
            repodata.add_dirstr(solvable.id, solv.SOLVABLE_FILELIST, dirname_id, file_name)

    def rpm_basic_deps(solvable, name, evr, arch):
        # Prv: $n . $a = $evr
        pool = solvable.repo.pool
        name_id = pool.str2id(name)
        evr_id = pool.str2id(evr)
        arch_id = pool.str2id(arch)
        rel = pool.rel2id(name_id, arch_id, solv.REL_ARCH)
        rel = pool.rel2id(rel, evr_id, solv.REL_EQ)
        solvable.add_deparray(solv.SOLVABLE_PROVIDES, rel)

    name = unit.get("name")
    solvable.name = name

    evr = libsolv_formatted_evr(unit.get("epoch"), unit.get("version"), unit.get("release"))
    solvable.evr = evr

    arch = unit.get("arch", "noarch")
    solvable.arch = arch

    vendor = unit.get("vendor")
    if vendor:
        vendor = vendor
        solvable.vendor = vendor

    for attribute_name in ("requires", "provides", "recommends"):
        for depunit in unit.get(attribute_name, []):
            rpm_dependency_conversion(solvable, depunit, attribute_name)

    rpm_filelist_conversion(solvable, unit)
    rpm_basic_deps(solvable, name, evr, arch)

    return solvable


def rpm_dependency_conversion(solvable, unit, attr_name, dependency_key=None):
    """Set the solvable dependencies.

    The dependencies of a unit are stored as a list of dictionaries,
    containing following values:
            name: <unit name> or a rich dep string; mandatory
            version: version of the dependency; optional
            epoch: epoch of the dependency; optional
            release: release of the dependency; optional
            flags: AND/OR; optional; if missing meaning by default AND

    These values are parsed by librpm.
    There are two cases how libsolv addresses the dependencies:

    * rich: the name of the dependency contains all required information:
        '(foo >= 1.0-3 AND bar != 0.9)'
        all the other attribute values are ignored

    * generic: the name, version, epoch, release and flags attributes
        are processed explicitly

    The dependency list is either of the provides, requires or the weak
    dependencies, the current case being stored under self.attr_name.

    Libsolv tracks a custom Dep object to represent a dependency of a
    solvable object; these are created in the pool object:

        dependency = pool.Dep('foo')

    The relationship to the solvable is tracked by a Rel pool object:

        relationship = pool.Rel(solv.REL_AND, pool.Dep(evr))

    where the evr is the 'epoch:version-release' string. The relationship
    is then recorded on the solvable explicitly by:

        solvable.add_deparray(solv.SOLVABLE_PROVIDES, relationship)

    If no explict relationship is provided in the flags attribute,
    the dependency can be used directly:

        solvable.add_deparray(solv.SOLVABLE_PROVIDES, dependency)

    Args:
        solvable (solvable): a libsolv solvable object
        unit (dict): the content unit to get the dependencies from

    """
    unit_name = unit[0]
    unit_flags = unit[1]
    unit_evr = libsolv_formatted_evr(unit[2], unit[3], unit[4])

    # e.g SOLVABLE_PROVIDES, SOLVABLE_REQUIRES...
    keyname = dependency_key or getattr(solv, "SOLVABLE_{}".format(attr_name.upper()))
    pool = solvable.repo.pool
    if unit_name.startswith("("):
        # the Rich/Boolean dependencies have just the 'name' attribute
        # this is always in the form: '(foo >= 1.2 with foo < 2.0)'
        dep = pool.parserpmrichdep(unit_name)
    else:
        # generic dependencies provide at least a solvable name
        dep = pool.Dep(unit_name)
        if unit_flags:
            # in case the flags unit attribute is populated, use it as
            # a solv.Rel object to denote solvable--dependency
            # relationship dependency in this case is a relationship
            # towards the dependency made from the 'flags', e.g:
            # solv.REL_EQ, and the evr fields
            if unit_flags == "EQ":
                rel_flags = solv.REL_EQ
            elif unit_flags == "LT":
                rel_flags = solv.REL_LT
            elif unit_flags == "GT":
                rel_flags = solv.REL_GT
            elif unit_flags == "LE":
                rel_flags = solv.REL_EQ | solv.REL_LT
            elif unit_flags == "GE":
                rel_flags = solv.REL_EQ | solv.REL_GT
            else:
                raise ValueError("Unsupported dependency flags %s" % unit_flags)
            dep = dep.Rel(rel_flags, pool.Dep(unit_evr))
    # register the constructed solvable dependency
    solvable.add_deparray(keyname, dep)


def module_to_solvable(solv_repo, unit):
    """Convert a Pulp Module in dict representation to a libsolv solvable.

    Args:
        solv_repo (solv.Repo): The repository the unit is being added into
        unit (pulp_rpm.plugins.models.Model): The unit being converted

    Returns: (solv.Solvable) The solvable created.
    """
    solvable = solv_repo.add_solvable()
    pool = solvable.repo.pool

    def module_solvable_name(unit):
        """
        Create a solvable name from module attributes.

        e.g. module:<name>:<stream>:<version>:<context>
        """
        return "module:{name}:{stream}:{version!s}:{context}".format(
            name=unit.get("name"),
            stream=unit.get("stream"),
            version=unit.get("version"),
            context=unit.get("context"),
        )

    def module_basic_deps(pool, solvable, solvable_name, name, stream, version, arch):
        """
        Create the basic module `Provides:` relations.
        """
        # Prv: module:$n:$s:$v:$c . $a
        solvable.nsvca_rel = pool.rel2id(
            pool.str2id(solvable_name), pool.str2id(arch), solv.REL_ARCH
        )
        solvable.add_deparray(solv.SOLVABLE_PROVIDES, solvable.nsvca_rel)

        # Prv: module()
        dep = pool.Dep("module()")
        solvable.add_deparray(solv.SOLVABLE_PROVIDES, dep)

        # Prv: module($n)
        dep_n = pool.Dep("module({})".format(name))
        solvable.add_deparray(solv.SOLVABLE_PROVIDES, dep_n)

        # Prv: module($n:$s)
        dep_ns = pool.Dep("module({}:{})".format(name, stream))
        solvable.add_deparray(solv.SOLVABLE_PROVIDES, dep_ns)

        # Prv: module($n:$s) = $v
        dep_ns_v = dep_n.Rel(solv.REL_EQ, pool.Dep(str(version)))
        solvable.add_deparray(solv.SOLVABLE_PROVIDES, dep_ns_v)

    def module_artifacts_conversion(pool, module_solvable, name, evr, arch):
        # propagate Req: module:$n:$s:$v:$c . $a to the modular RPM i.e
        # make the rpm require this module
        name_id = pool.str2id(name)
        evr_id = pool.str2id(evr)
        arch_id = pool.str2id(arch)

        # $n.$a = $evr
        rel = pool.rel2id(name_id, arch_id, solv.REL_ARCH)
        rel = pool.rel2id(rel, evr_id, solv.REL_EQ)
        selection = pool.matchdepid(
            rel, solv.SOLVABLE_NAME | solv.SOLVABLE_ARCH | solv.SOLVABLE_EVR, solv.SOLVABLE_PROVIDES
        )

        for rpm_solvable in selection.solvables():
            # Make the artifact require this module
            rpm_solvable.add_deparray(solv.SOLVABLE_REQUIRES, module_solvable.nsvca_rel)
            # Provide: modular-package()
            rpm_solvable.add_deparray(solv.SOLVABLE_PROVIDES, pool.Dep("modular-package()"))

    solvable_name = module_solvable_name(unit)
    solvable.name = solvable_name
    solvable.evr = ""

    arch = unit.get("arch", "noarch")
    solvable.arch = arch

    name = unit.get("name")
    stream = unit.get("stream")
    version = unit.get("version")

    module_basic_deps(pool, solvable, solvable_name, name, stream, version, arch)

    for artifact in unit.get("artifacts", []):
        nevra_tuple = parse_nevra(artifact)
        artifact_name = nevra_tuple[0]
        artifact_epoch = nevra_tuple[1]
        artifact_version = nevra_tuple[2]
        artifact_release = nevra_tuple[3]
        artifact_arch = nevra_tuple[4] if nevra_tuple[4] else "noarch"
        artifact_evr = libsolv_formatted_evr(artifact_epoch, artifact_version, artifact_release)

        module_artifacts_conversion(pool, solvable, artifact_name, artifact_evr, artifact_arch)

    module_dependencies_conversion(pool, solvable, unit.get("dependencies", {}))
    pool.createwhatprovides()  # TODO: It would be great to do this less often

    return solvable


def module_dependencies_conversion(pool, module_solvable, dependency_list):
    """
    Process the module dependency list.

    So for example for following input:
        dependency_list = [{'gtk': ['1'], 'foo': ['1']}]

    The resulting solv.Dep expression will be:
        ((module(gtk) with module(gtk:1)) and (module(foo) with module(foo:1)))

    This Dep expression is then applied to the REQUIRES: deparray of the module solvable.

    Args:
        pool (solv.Pool): The libsolv pool that owns the module
        module_solvable (solv.Solvable): A solvable representing the module
        dependency_list (list): List of dictionaries representing modulemd dependency data.
    """
    # A near exact copy of the algorithm here:
    # https://pagure.io/fm-orchestrator/blob/db03f0a7f530cc2bf2f8971f085a9e6b71595d70/f/
    # module_build_service/mmd_resolver.py#_53

    def stream_dep(name, stream):
        """
        Create a libsolv Dep from a name and stream.

        Every name:stream combination from dict in `deps` list is expressed as `solv.Dep`
        instance and is represented internally in solv with "module(name:stream)".
        This is parallel to RPM-world "Provides: perl(foo)" or "Requires: perl(foo)",
        but in this method, we are only constructing the condition after the "Provides:"
        or "Requires:". This method creates such solve.Dep().
        """
        return pool.Dep("module({}:{})".format(name, stream))

    def dep_or_rel(dep, op, rel):
        """
        Helper method for creating reps and relational deps.

        There are relations between modules in `deps`. For example:
          deps = [{'gtk': ['1'], 'foo': ['1']}]" means "gtk:1 and foo:1" are both required.
          deps = [{'gtk': ['1', '2']}"] means "gtk:1 or gtk:2" are required.

        This method helps creating such relations using following syntax:
          dep_or_rel(solv.Dep, solve.REL_OR, stream_dep(name, stream))
          dep_or_rel(solv.Dep, solve.REL_AND, stream_dep(name, stream))
          dep_or_rel(solv.Dep, solve.REL_WITH, stream_dep(name, stream))
          dep_or_rel(solv.Dep, solve.REL_WITHOUT, stream_dep(name, stream))
        """
        dep.Rel(op, rel) if dep is not None else rel

    # Check each dependency dict in dependency_list and generate the solv requirements.
    reqs = None
    for dep_dict in dependency_list:
        require = None
        for name, streams in dep_dict.items():
            if name == "platform":
                # no need to fake the platform (streams) later on
                continue

            # The req_pos will store solv.Dep expression for "positive" requirements.
            # That is the case of 'gtk': ['1', '2'].
            # The req_neg will store negative requirements like 'gtk': ['-1', '-2'].
            req_pos = req_neg = None

            # For each stream in `streams` for this dependency, generate the
            # module(name:stream) solv.Dep and add REL_OR relations between them.
            for stream in streams:
                if stream.startswith("-"):
                    req_neg = dep_or_rel(req_neg, solv.REL_OR, stream_dep(name, stream[1:]))
                else:
                    req_pos = dep_or_rel(req_pos, solv.REL_OR, stream_dep(name, stream))

            # Generate the module(name) solv.Dep.
            req = pool.Dep("module({})".format(name))

            # Use the REL_WITH for positive requirements and REL_WITHOUT for negative
            # requirements.
            if req_pos is not None:
                req = req.Rel(solv.REL_WITH, req_pos)
            elif req_neg is not None:
                req = req.Rel(solv.REL_WITHOUT, req_neg)

            # And in the end use AND between the last name:[streams] and the current one.
            require = dep_or_rel(require, solv.REL_AND, req)

        # NOTE: In the original algorithm, this was an OR operation. We don't want only one
        # set of deps (for one platform), we want the deps for all platforms. Hence, solv.REL_AND.
        reqs = dep_or_rel(reqs, solv.REL_AND, require)

    module_solvable.add_deparray(solv.SOLVABLE_REQUIRES, reqs)


def module_defaults_unit_to_solvable(solv_repo, unit):
    """Convert a Pulp Module Default dict to a libsolv solvable.

    Args:
        solv_repo (solv.Repo): The repository the unit is being added into.
        unit (pulp_rpm.plugins.models.Model): The unit being converted.

    Returns: (solv.Solvable) The solvable created.
    """
    solvable = solv_repo.add_solvable()
    solvable.evr = ""
    # a module default has no arch, use 'noarch'
    solvable.arch = "noarch"

    name = unit.get("name")
    stream = unit.get("stream")

    solvable.name = "module-default:{}".format(name)

    pool = solvable.repo.pool

    def module_defaults_basic_deps(solvable, name, stream):
        """
        Links a module and its default with dependencies.
        """
        # we are making all modules require the module-default regardless of they are default
        # since the module-default can cary profile information
        module_depid = pool.Dep("module({})".format(name), 0)

        module_default_depid = pool.Dep(solvable.name)
        if not module_depid:
            return

        # tell libsolv that this solvable provides the module-default for the module name
        solvable.add_deparray(solv.SOLVABLE_PROVIDES, module_default_depid)

        pool.createwhatprovides()
        for module in pool.whatprovides(module_depid):
            # module default metadata doesn't have to specify a stream, we only want to make
            # module:name:stream:{ver}:{ctx} provide the default when it does. However, in either
            # case we want the modules to require the module-default because it can carry
            # important profile information.
            if stream:
                # mark the related module so it can be queried as
                # '(module() with module-default())' i.e such that it's easy visible through
                # a pool.whatprovides that it has a default
                module.add_deparray(solv.SOLVABLE_PROVIDES, pool.Dep("module-default()"))

        # Note: Since we're copying the module default metadata as-is without modification or
        # regeneration, that means that "profiles" may be copied for streams that do not exist.
        # We think this is probably OK but if it is not, the solution is to "require" all streams
        # for which a profile exists.

    module_defaults_basic_deps(solvable, name, stream)

    return solvable


class UnitSolvableMapping:
    """Map libsolv solvables to Pulp units and repositories.

    Translate between what libsolv understands, solvable IDs in a pool, and what Pulp understands,
    units and repositories.
    """

    def __init__(self):
        """Mapping Init."""
        # Stores data in the form (pulp_unit_id, pulp_repo_id): solvable
        self._mapping_unit_to_solvable = {}
        # Stores data in the form solvable_id: (pulp_unit_id, pulp_repo_id)
        self._mapping_solvable_to_unit = {}
        # Stores data in the form pulp_repo_id: libsolv_repo_id
        self._mapping_repos = {}

    def register(self, unit_id, solvable, repo_id):
        """Store the matching of a unit-repo pair to a solvable inside of the mapping."""
        if not self.get_repo(str(repo_id)):
            raise ValueError(
                "Attempting to register unit {} to unregistered repo {}".format(unit_id, repo_id)
            )

        self._mapping_solvable_to_unit.setdefault(solvable.id, (unit_id, repo_id))
        self._mapping_unit_to_solvable.setdefault((unit_id, repo_id), solvable)

        logger.debug(
            "Loaded unit {unit}, {repo} as {solvable}".format(
                unit=unit_id, solvable=solvable, repo=repo_id
            )
        )

    def register_repo(self, repo_id, libsolv_repo):
        """Store the repo (Pulp) - repo (libsolv) pair."""
        return self._mapping_repos.setdefault(str(repo_id), libsolv_repo)

    def get_repo(self, repo_id):
        """Return the repo from the mapping."""
        return self._mapping_repos.get(repo_id)

    def get_unit_id(self, solvable):
        """Get the (unit, repo_id) pair for a given solvable."""
        return self._mapping_solvable_to_unit.get(solvable.id)

    def get_solvable(self, unit_id, repo_id):
        """Fetch the libsolv solvable associated with a unit-repo pair."""
        return self._mapping_unit_to_solvable.get((unit_id, repo_id))

    def get_repo_units(self, repo_id):
        """Get back unit ids of all units that were in a repo based on the mapping."""
        return set(
            unit_id
            for (unit_id, unit_repo_id) in self._mapping_unit_to_solvable.keys()
            if unit_repo_id == repo_id
        )

    def get_units_from_solvables(self, solvables):
        """Map a list of solvables into their Pulp units, keyed by the repo they came from."""
        repo_unit_map = collections.defaultdict(set)
        for solvable in solvables:
            (unit_id, repo_id) = self.get_unit_id(solvable)
            repo_unit_map[repo_id].add(unit_id)

        return repo_unit_map


class Solver:
    """A Solver object that can speak in terms of Pulp units."""

    def __init__(self):
        """Solver Init."""
        self._finalized = False
        self._pool = solv.Pool()
        self._pool.setarch()  # prevent https://github.com/openSUSE/libsolv/issues/267
        self._pool.set_flag(solv.Pool.POOL_FLAG_IMPLICITOBSOLETEUSESCOLORS, 1)
        self.mapping = UnitSolvableMapping()

    def finalize(self):
        """Finalize the solver - a finalized solver is ready for depsolving.

        Libsolv needs to perform some scans/hashing operations before it can do certain things.
        For more details see:
        https://github.com/openSUSE/libsolv/blob/master/doc/libsolv-bindings.txt
        """
        self._pool.installed = self.mapping.get_repo(COMBINED_TARGET_REPO_NAME)
        self._pool.addfileprovides()
        self._pool.createwhatprovides()
        self._finalized = True

    def load_source_repo(self, repo_version):
        """Load the provided Pulp repo as a source repo.

        All units in the repo will be available to be "installed", or copied.
        """
        libsolv_repo_name = self._load_from_version(repo_version)
        logger.debug(
            "Loaded repository '{}' version '{}' as source repo".format(
                repo_version.repository, repo_version.number
            )
        )
        return libsolv_repo_name

    def load_target_repo(self, repo_version):
        """Load the provided Pulp repo into the combined target repo.

        All units in the repo will be added to the combined target repo, the contents of which
        are considered "installed" by the solver.
        """
        libsolv_repo_name = self._load_from_version(repo_version, as_target=True)
        logger.debug(
            "Loaded repository '{}' version '{}' into combined target repo".format(
                repo_version.repository, repo_version.number
            )
        )
        return libsolv_repo_name

    def _repo_version_to_libsolv_name(self, repo_version):
        """Produce a name to use for the libsolv repo from the repo version."""
        return "{}: version={}".format(repo_version.repository.name, repo_version.number)

    def _load_from_version(self, repo_version, as_target=False):
        """
        Generate solvables from Pulp units and add them to the mapping.

        In some circumstances, we want to load multiple Pulp "repos" together into one libsolv
        "repo", because libsolv can only have one repo be "installed" at a time. Therefore, when
        the override_repo_name is specified, the created solvables are associated with the
        override repo, but the mapping stores them with their original Pulp repo_id.

        Args:
            override_repo_name (str): Override name to use when adding solvables to a libsolv repo
        """
        if as_target:
            libsolv_repo_name = COMBINED_TARGET_REPO_NAME
        else:
            libsolv_repo_name = self._repo_version_to_libsolv_name(repo_version)

        repo = self.mapping.get_repo(libsolv_repo_name)
        if not repo:
            repo = self.mapping.register_repo(
                libsolv_repo_name, self._pool.add_repo(libsolv_repo_name)
            )
            repodata = repo.add_repodata()
        else:
            repodata = repo.first_repodata()

        # Load packages into the solver

        package_ids = repo_version.content.filter(pulp_type=models.Package.get_pulp_type()).only(
            "pk"
        )

        nonmodular_rpms = models.Package.objects.filter(
            pk__in=package_ids, is_modular=False
        ).values(*RPM_FIELDS)

        for rpm in nonmodular_rpms.iterator(chunk_size=5000):
            self._add_unit_to_solver(rpm_to_solvable, rpm, repo, libsolv_repo_name)

        modular_rpms = models.Package.objects.filter(pk__in=package_ids, is_modular=True).values(
            *RPM_FIELDS
        )

        for rpm in modular_rpms.iterator(chunk_size=5000):
            self._add_unit_to_solver(rpm_to_solvable, rpm, repo, libsolv_repo_name)

        # Load modules into the solver

        module_ids = repo_version.content.filter(pulp_type=models.Modulemd.get_pulp_type()).only(
            "pk"
        )

        modules = models.Modulemd.objects.filter(pk__in=module_ids).values(*MODULE_FIELDS)

        for module in modules.iterator(chunk_size=5000):
            self._add_unit_to_solver(module_to_solvable, module, repo, libsolv_repo_name)

        # Load module defaults into the solver

        module_defaults_ids = repo_version.content.filter(
            pulp_type=models.ModulemdDefaults.get_pulp_type()
        ).only("pk")

        modulemd_defaults = models.ModulemdDefaults.objects.filter(
            pk__in=module_defaults_ids
        ).values(*MODULE_DEFAULTS_FIELDS)

        for module_default in modulemd_defaults.iterator(chunk_size=5000):
            self._add_unit_to_solver(
                module_defaults_unit_to_solvable, module_default, repo, libsolv_repo_name
            )

        # Need to call pool->addfileprovides(), pool->createwhatprovides() after loading new repo
        self._finalized = False

        repodata.internalize()
        return libsolv_repo_name

    def _add_unit_to_solver(self, conversion_func, unit, repo, libsolv_repo_name):
        solvable = conversion_func(repo, unit)
        self.mapping.register(unit["pk"], solvable, libsolv_repo_name)

    def _build_warnings(self, problems):
        """Builds a list of 'warnable' depsolving errors.

        "install" problems aren't relevant to "is the repo-version resulting from this copy
        dependency-complete", so we choose to ignore them, as they only hide 'real'
        depsolving issues while alarming the user.

        Args:
            problems: (list) List of libsolv.Problem from latest depsolving attempt
        Returns: (list) List of error-strings for 'warnable' problems.
        """
        warn_on = [
            solv.Solver.SOLVER_RULE_JOB_NOTHING_PROVIDES_DEP,
            solv.Solver.SOLVER_RULE_JOB_UNKNOWN_PACKAGE,
            solv.Solver.SOLVER_RULE_PKG,
        ]
        warnings = []
        for problem in problems:
            for problem_rule in problem.findallproblemrules():
                for info in problem_rule.allinfos():
                    if info.type in warn_on:
                        warnings.append(str(info))
        return warnings

    def resolve_dependencies(self, unit_repo_map):
        """Resolve the total set of packages needed for the packages passed in, as DNF would.

        Find the set of dependent units and return them in a dictionary where
        the key is the repository the set of units came from.

        Find the RPM dependencies that need to be copied to satisfy copying the provided units,
        taking into consideration what units are already present in the target repository.

        Create libsolv jobs to install each one of the units passed in, collect and combine the
        results. For modules, libsolv jobs are created to install each of their artifacts
        separately.

        A libsolv "Job" is a request for the libsolv sat solver to process. For instance a job with
        the flags SOLVER_INSTALL | SOLVER_SOLVABLE will tell libsolv to solve an installation of
        a package which is specified by solvable ID, as opposed to by name, or by pattern, which
        are other options available.

        See: https://github.com/openSUSE/libsolv/blob/master/doc/libsolv-bindings.txt#the-job-class

        In the context of this use of libsolv, we have added/are taking advantage of the flag
        solv.Pool.POOL_FLAG_IMPLICITOBSOLETEUSESCOLORS. to handle multiarch repos correctly.

        Args:
            unit_repo_map: (dict) An iterable oflibsolv_repo_name =

        Returns: (dict) A dictionary of form {'repo_id': set(unit_ids**)}
        """
        assert self._finalized, "Depsolver must be finalized before it can be used"

        solvables = []
        # units (like advisories) that aren't solved for in the dependency solver need to be
        # passed through to the end somehow, so let's add them to a second mapping that mirrors
        # the first and combine them again at the end.
        passthrough = {k: set() for k in unit_repo_map.keys()}
        for repo, units in unit_repo_map.items():
            for unit in units:
                if unit.pulp_type in {"rpm.package", "rpm.modulemd", "rpm.modulemd_defaults"}:
                    solvables.append(self.mapping.get_solvable(unit.pk, repo))
                passthrough[repo].add(unit.pk)

        self._pool.createwhatprovides()
        flags = solv.Job.SOLVER_INSTALL | solv.Job.SOLVER_SOLVABLE

        solvables_to_copy = set(solvables)
        result_solvables = set()
        install_jobs = []

        while solvables_to_copy:
            # Take one solvable
            solvable = solvables_to_copy.pop()

            if solvable.name.startswith("module:"):
                # If the solvable being installed is a module, try to install it and all of its
                # modular artifact dependencies
                module_dep = self._pool.rel2id(
                    self._pool.str2id(solvable.name),
                    self._pool.str2id(solvable.arch),
                    solv.REL_ARCH,
                )
                module_artifacts = self._pool.whatcontainsdep(solv.SOLVABLE_REQUIRES, module_dep)

                module_install_job = self._pool.Job(flags, solvable.id)
                install_jobs.append(module_install_job)

                for artifact in module_artifacts:
                    artifact_install_job = self._pool.Job(flags, artifact.id)
                    install_jobs.append(artifact_install_job)
            else:
                # If the unit being copied is not a module, just install it alone
                unit_install_job = self._pool.Job(flags, solvable.id)
                install_jobs.append(unit_install_job)

        # Take a list of jobs, get a solution, return the set of solvables that needed to
        # be installed.
        solver = self._pool.Solver()
        solver.set_flag(solv.Solver.SOLVER_FLAG_FOCUS_INSTALLED, 1)

        raw_problems = solver.solve(install_jobs)
        # The solver is simply ignoring the problems encountered and proceeds associating
        # any new solvables/units. This might be reported back to the user one day over
        # the REST API. For now, log only "real" dependency issues (typically some variant
        # of "can't find the package"
        dependency_warnings = self._build_warnings(raw_problems)
        if dependency_warnings:
            logger.warning(
                "Encountered problems solving dependencies, "
                "copy may be incomplete: {}".format(", ".join(dependency_warnings))
            )

        transaction = solver.transaction()
        if settings.SOLVER_DEBUG_LOGS:
            write_solver_debug_data(solver, raw_problems, self.mapping, full=False)
        result_solvables.update(set(transaction.newsolvables()))

        solved_units = self.mapping.get_units_from_solvables(result_solvables)
        for k in unit_repo_map.keys():
            solved_units[k] |= passthrough[k]

        return solved_units


# Descriptions copied from the libsolv documentation
# https://github.com/openSUSE/libsolv/blob/master/doc/libsolv-bindings.txt
def write_solver_debug_data(solver, problems, mapping, full=False):
    """Dump the state of the solver including actions decided upon and problems encountered."""
    from pulpcore.plugin.models import Task
    from pathlib import Path

    debugdata_dir = Path("/var/tmp/pulp") / str(Task.current().pulp_id)
    debugdata_dir.mkdir(parents=True, exist_ok=True)
    logger.info("Writing solver debug data to {}".format(debugdata_dir))

    transaction = solver.transaction()
    summary_path = debugdata_dir / "depsolving_summary.txt"

    reason_desc_map = {
        solv.Solver.SOLVER_REASON_UNRELATED: (
            "SOLVER_REASON_UNRELATED",
            "The package status did not change as it was not related to any job.",
        ),
        solv.Solver.SOLVER_REASON_UNIT_RULE: (
            "SOLVER_REASON_UNIT_RULE",
            "The package was installed/erased/kept because of a unit rule, "
            "i.e. a rule where all literals but one were false.",
        ),
        solv.Solver.SOLVER_REASON_KEEP_INSTALLED: (
            "SOLVER_REASON_KEEP_INSTALLED",
            "The package was chosen when trying to keep as many packages installed as possible.",
        ),
        solv.Solver.SOLVER_REASON_RESOLVE_JOB: (
            "SOLVER_REASON_RESOLVE_JOB",
            "The decision happened to fulfill a job rule.",
        ),
        solv.Solver.SOLVER_REASON_UPDATE_INSTALLED: (
            "SOLVER_REASON_UPDATE_INSTALLED",
            "The decision happened to fulfill a package update request.",
        ),
        solv.Solver.SOLVER_REASON_RESOLVE: (
            "SOLVER_REASON_RESOLVE",
            "The package was installed to fulfill package dependencies.",
        ),
        solv.Solver.SOLVER_REASON_WEAKDEP: (
            "SOLVER_REASON_WEAKDEP",
            "The package was installed because of a weak dependency (Recommends or Supplements).",
        ),
        solv.Solver.SOLVER_REASON_RECOMMENDED: (
            "SOLVER_REASON_RECOMMENDED",
            "The package was installed because of a weak dependency (Recommends or Supplements).",
        ),
        solv.Solver.SOLVER_REASON_SUPPLEMENTED: (
            "SOLVER_REASON_SUPPLEMENTED",
            "The package was installed because of a weak dependency (Recommends or Supplements).",
        ),
    }

    rule_desc_map = {
        solv.Solver.SOLVER_RULE_UNKNOWN: (
            "SOLVER_RULE_UNKNOWN",
            "A rule of an unknown class. You should never encounter those.",
        ),
        solv.Solver.SOLVER_RULE_PKG: ("SOLVER_RULE_PKG", "A package dependency rule."),
        solv.Solver.SOLVER_RULE_UPDATE: (
            "SOLVER_RULE_UPDATE",
            "A rule to implement the update policy of installed packages. Every installed "
            "package has an update rule that consists of the packages that may replace the "
            "installed package.",
        ),
        solv.Solver.SOLVER_RULE_FEATURE: (
            "SOLVER_RULE_FEATURE",
            "Feature rules are fallback rules used when an update rule is disabled. They "
            "include all packages that may replace the installed package ignoring the update "
            "policy, i.e. they contain downgrades, arch changes and so on. Without them, the "
            "solver would simply erase installed packages if their update rule gets disabled.",
        ),
        solv.Solver.SOLVER_RULE_JOB: (
            "SOLVER_RULE_JOB",
            "Job rules implement the job given to the solver.",
        ),
        solv.Solver.SOLVER_RULE_DISTUPGRADE: (
            "SOLVER_RULE_DISTUPGRADE",
            "These are simple negative assertions that make sure that only packages are kept "
            "that are also available in one of the repositories.",
        ),
        solv.Solver.SOLVER_RULE_INFARCH: (
            "SOLVER_RULE_INFARCH",
            "Infarch rules are also negative assertions, they disallow the installation of "
            "packages when there are packages of the same name but with a better architecture.",
        ),
        solv.Solver.SOLVER_RULE_CHOICE: (
            "SOLVER_RULE_CHOICE",
            "Choice rules are used to make sure that the solver prefers updating to installing "
            "different packages when some dependency is provided by multiple packages with "
            "different names. The solver may always break choice rules, so you will not see them "
            "when a problem is found.",
        ),
        solv.Solver.SOLVER_RULE_LEARNT: (
            "SOLVER_RULE_LEARNT",
            "These rules are generated by the solver to keep it from running into the same "
            "problem multiple times when it has to backtrack. They are the main reason why "
            "a sat solver is faster than other dependency solver implementations.",
        ),
        # Special dependency rule types:
        solv.Solver.SOLVER_RULE_PKG_NOT_INSTALLABLE: (
            "SOLVER_RULE_PKG_NOT_INSTALLABLE",
            "This rule was added to prevent the installation of a package of an architecture "
            "that does not work on the system.",
        ),
        solv.Solver.SOLVER_RULE_PKG_NOTHING_PROVIDES_DEP: (
            "SOLVER_RULE_PKG_NOTHING_PROVIDES_DEP",
            "The package contains a required dependency which was not provided by any package.",
        ),
        solv.Solver.SOLVER_RULE_PKG_REQUIRES: (
            "SOLVER_RULE_PKG_REQUIRES",
            "Similar to SOLVER_RULE_PKG_NOTHING_PROVIDES_DEP, but in this case some packages "
            "provided the dependency but none of them could be installed due to other "
            "dependency issues.",
        ),
        solv.Solver.SOLVER_RULE_PKG_SELF_CONFLICT: (
            "SOLVER_RULE_PKG_SELF_CONFLICT",
            "The package conflicts with itself. This is not allowed by older rpm versions.",
        ),
        solv.Solver.SOLVER_RULE_PKG_CONFLICTS: (
            "SOLVER_RULE_PKG_CONFLICTS",
            "To fulfill the dependencies two packages need to be installed, but one of the "
            "packages contains a conflict with the other one.",
        ),
        solv.Solver.SOLVER_RULE_PKG_SAME_NAME: (
            "SOLVER_RULE_PKG_SAME_NAME",
            "The dependencies can only be fulfilled by multiple versions of a package, but "
            "installing multiple versions of the same package is not allowed.",
        ),
        solv.Solver.SOLVER_RULE_PKG_OBSOLETES: (
            "SOLVER_RULE_PKG_OBSOLETES",
            "To fulfill the dependencies two packages need to be installed, but one of the "
            "packages obsoletes the other one.",
        ),
        solv.Solver.SOLVER_RULE_PKG_IMPLICIT_OBSOLETES: (
            "SOLVER_RULE_PKG_IMPLICIT_OBSOLETES",
            "To fulfill the dependencies two packages need to be installed, but one of the "
            "packages has provides a dependency that is obsoleted by the other one. See the "
            "POOL_FLAG_IMPLICITOBSOLETEUSESPROVIDES flag.",
        ),
        solv.Solver.SOLVER_RULE_PKG_INSTALLED_OBSOLETES: (
            "SOLVER_RULE_PKG_INSTALLED_OBSOLETES",
            "To fulfill the dependencies a package needs to be installed that is obsoleted "
            "by an installed package. See the POOL_FLAG_NOINSTALLEDOBSOLETES flag.",
        ),
        solv.Solver.SOLVER_RULE_JOB_NOTHING_PROVIDES_DEP: (
            "SOLVER_RULE_JOB_NOTHING_PROVIDES_DEP",
            "The user asked for installation of a package providing a specific dependency, but "
            "no available package provides it.",
        ),
        solv.Solver.SOLVER_RULE_JOB_UNKNOWN_PACKAGE: (
            "SOLVER_RULE_JOB_UNKNOWN_PACKAGE",
            "The user asked for installation of a package with a specific name, but no available "
            "package has that name.",
        ),
        solv.Solver.SOLVER_RULE_JOB_PROVIDED_BY_SYSTEM: (
            "SOLVER_RULE_JOB_PROVIDED_BY_SYSTEM",
            "The user asked for the erasure of a dependency that is provided by the system "
            "(i.e. for special hardware or language dependencies), this cannot be done with "
            "a job.",
        ),
        solv.Solver.SOLVER_RULE_JOB_UNSUPPORTED: (
            "SOLVER_RULE_JOB_UNSUPPORTED",
            "The user asked for something that is not yet implemented, e.g. the installation "
            "of all packages at once.",
        ),
    }

    with summary_path.open("wt") as summary:

        print("Problems Encountered:", file=summary)
        print("=====================", file=summary)
        for problem in problems:
            print(str(problem), file=summary)
        print(file=summary)

        print("Packages transferred:", file=summary)
        print("=====================", file=summary)
        print(file=summary)

        for solvable in transaction.newsolvables():
            (reason, rule) = solver.describe_decision(solvable)

            print(
                "{name}-{evr}.{arch}".format(
                    name=solvable.name, evr=solvable.evr, arch=solvable.arch
                ),
                file=summary,
            )

            (reason_name, reason_description) = reason_desc_map[reason]
            (unit_id, from_repo) = mapping.get_unit_id(solvable)
            print(
                "    Pulp Content unit '{}' from repo '{}'".format(unit_id, from_repo), file=summary
            )
            print("    Reason: {} - {}".format(reason_name, reason_description), file=summary)
            print("    Rules:", file=summary)
            for info in rule.allinfos():
                (rule_name, rule_description) = rule_desc_map[info.type]
                print("        {} - {}".format(rule_name, rule_description), file=summary)
                if info.solvable:
                    pkg = str(info.solvable)
                    dep = str(info.dep)
                    print(
                        "            Because package '{}' requires '{}'".format(pkg, dep),
                        file=summary,
                    )

            print(file=summary)

    if full:
        solver.write_testcase(str(debugdata_dir))



from logging import getLogger

from django.db import models

from pulpcore.plugin.models import Content

from pulp_rpm.app.models.package import Package


log = getLogger(__name__)


class Modulemd(Content):
    """
    The "Modulemd" content type. Modularity support.

    Fields:
        name (Text):
            Name of the modulemd
        stream (Text):
            The modulemd's stream
        version (Text):
            The version of the modulemd.
        static_context (Boolean):
            If True, then the context flag is a string of up to thirteen [a-zA-Z0-9_] characters
            representing a build and runtime configuration for this stream. If False or unset, then
            the context flag is filled in by the buildsystem with a short hash of the module's
            NSV and expanded dependencies.
        context (Text):
            The context flag serves to distinguish module builds with the
            same name, stream and version and plays an important role in
            future automatic module stream name expansion.
        arch (Text):
            Module artifact architecture.
        dependencies (Text):
            Module dependencies, if any.
        artifacts (Text):
            Artifacts shipped with this module.
        packages (Text):
            List of Packages connected to this modulemd.
        snippet (Text):
            A string to hold modulemd-obsolete snippet
    """

    TYPE = "modulemd"

    # required metadata
    name = models.TextField()
    stream = models.TextField()
    version = models.TextField()
    context = models.TextField()
    arch = models.TextField()

    static_context = models.BooleanField(null=True)
    dependencies = models.JSONField(default=list)
    artifacts = models.JSONField(default=list)
    packages = models.ManyToManyField(Package)
    profiles = models.JSONField(default=dict)
    description = models.TextField()

    snippet = models.TextField()

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"
        unique_together = ("name", "stream", "version", "context", "arch")


class ModulemdDefaults(Content):
    """
    The "Modulemd Defaults" content type. Modularity support.

    Fields:
        module (Text):
            Modulemd name.
        stream (Text):
            Modulemd default stream.
        profiles (Text):
            Default profiles for modulemd streams.
        digest (Text):
            Modulemd digest
        snippet (Text):
            A string to hold modulemd-obsolete snippet
    """

    TYPE = "modulemd_defaults"

    module = models.TextField()
    stream = models.TextField()
    profiles = models.JSONField(default=list)
    digest = models.TextField(unique=True)

    snippet = models.TextField()

    repo_key_fields = ("module",)

    @classmethod
    def natural_key_fields(cls):
        """
        Digest is used as a natural key for ModulemdDefaults.
        """
        return ("digest",)

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"


class ModulemdObsolete(Content):
    """
    The "Modulemd Obsoletes" content type.

    Fields:
        modified (models.DateTimeField):
            A DateTime field representing last modification of modulemd obsolete.
        reset (Bool):
            A boolean option to reset all previously specified obsoletes
        module (Text):
            A string representing a Name of a module that is EOLed
        stream (Text):
            A string representing a Stream of a module that is EOLed
        context (Text):
            A string representing a Context of a module that is EOLed
        eol_date (models.DateTimeField):
            A DateTime field representing end of life date.
        message (Text):
            A string describing the change, reason, etc.
        obsolete_by (JSON):
            A dict to provide details about the obsoleting module and stream
        snippet (Text):
            A string to hold modulemd-obsolete snippet
    """

    TYPE = "modulemd_obsolete"

    # Mandatory fields
    modified = models.DateTimeField()
    module_name = models.TextField()
    module_stream = models.TextField()
    message = models.TextField()

    # Optional fields
    override_previous = models.BooleanField(null=True)
    module_context = models.TextField(null=True)
    eol_date = models.DateTimeField(null=True)
    obsoleted_by_module_name = models.TextField(null=True)
    obsoleted_by_module_stream = models.TextField(null=True)

    snippet = models.TextField()

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"
        unique_together = ("modified", "module_name", "module_stream")



"""
Tests PulpImporter and PulpImport functionality.

NOTE: assumes ALLOWED_EXPORT_PATHS and ALLOWED_IMPORT_PATHS settings contain "/tmp" - all tests
will fail if this is not the case.
"""
from pulp_smash import api, cli, config
from pulp_smash.utils import uuid4
from pulp_smash.pulp3.bindings import (
    delete_orphans,
    monitor_task,
    monitor_task_group,
    PulpTestCase,
)
from pulp_smash.pulp3.utils import gen_repo

from pulp_rpm.tests.functional.constants import RPM_KICKSTART_FIXTURE_URL, RPM_UNSIGNED_FIXTURE_URL
from pulp_rpm.tests.functional.utils import (
    gen_rpm_client,
    gen_rpm_remote,
)

from pulpcore.client.pulpcore import (
    ApiClient as CoreApiClient,
    ExportersPulpApi,
    ExportersPulpExportsApi,
    ImportersPulpApi,
    ImportersPulpImportsApi,
)
from pulpcore.client.pulp_rpm import (
    ContentDistributionTreesApi,
    RepositoriesRpmApi,
    RpmRepositorySyncURL,
    RemotesRpmApi,
)

NUM_REPOS = 2


class PulpImportTestBase(PulpTestCase):
    """
    Base functionality for PulpImporter and PulpImport test classes.
    """

    @classmethod
    def _setup_repositories(cls, url=None):
        """Create and sync a number of repositories to be exported."""
        # create and remember a set of repo
        import_repos = []
        export_repos = []
        remotes = []
        for r in range(NUM_REPOS):
            import_repo = cls.repo_api.create(gen_repo())
            export_repo = cls.repo_api.create(gen_repo())

            if url:
                body = gen_rpm_remote(url)
            else:
                body = gen_rpm_remote()
            remote = cls.remote_api.create(body)

            repository_sync_data = RpmRepositorySyncURL(remote=remote.pulp_href)
            sync_response = cls.repo_api.sync(export_repo.pulp_href, repository_sync_data)
            monitor_task(sync_response.task)
            # remember it
            export_repos.append(export_repo)
            import_repos.append(import_repo)
            remotes.append(remote)
        return import_repos, export_repos, remotes

    @classmethod
    def _create_exporter(cls, cleanup=True):
        body = {
            "name": uuid4(),
            "repositories": [r.pulp_href for r in cls.export_repos],
            "path": "/tmp/{}".format(uuid4()),
        }
        exporter = cls.exporter_api.create(body)
        return exporter

    @classmethod
    def _create_export(cls):
        export_response = cls.exports_api.create(cls.exporter.pulp_href, {})
        monitor_task(export_response.task)
        task = cls.client.get(export_response.task)
        resources = task["created_resources"]
        export_href = resources[0]
        export = cls.exports_api.read(export_href)
        return export

    @classmethod
    def _create_chunked_export(cls):
        export_response = cls.exports_api.create(cls.exporter.pulp_href, {"chunk_size": "5KB"})
        monitor_task(export_response.task)
        task = cls.client.get(export_response.task)
        resources = task["created_resources"]
        export_href = resources[0]
        export = cls.exports_api.read(export_href)
        return export

    @classmethod
    def _delete_exporter(cls, delete_file=True):
        """
        Utility routine to delete an exporter.

        Also removes the export-directory and all its contents.
        """
        if delete_file:
            cli_client = cli.Client(cls.cfg)
            cmd = ("rm", "-rf", cls.exporter.path)
            cli_client.run(cmd, sudo=True)
        cls.exporter_api.delete(cls.exporter.pulp_href)

    @classmethod
    def tearDownClass(cls):
        """Clean up."""
        for remote in cls.remotes:
            cls.remote_api.delete(remote.pulp_href)

        for repo in cls.export_repos:
            cls.repo_api.delete(repo.pulp_href)

        for repo in cls.import_repos:
            cls.repo_api.delete(repo.pulp_href)

        cls._delete_exporter()

        delete_orphans()

    def _create_importer(self, name=None, cleanup=True):
        """Create an importer."""
        mapping = {}
        if not name:
            name = uuid4()

        for idx, repo in enumerate(self.export_repos):
            mapping[repo.name] = self.import_repos[idx].name

        body = {
            "name": name,
            "repo_mapping": mapping,
        }

        importer = self.importer_api.create(body)

        if cleanup:
            self.addCleanup(self.importer_api.delete, importer.pulp_href)

        return importer

    def _perform_import(self, importer, chunked=False):
        """Perform an import with importer."""
        if chunked:
            filenames = [
                f for f in list(self.chunked_export.output_file_info.keys()) if f.endswith("json")
            ]
            import_response = self.imports_api.create(importer.pulp_href, {"toc": filenames[0]})
        else:
            filenames = [
                f for f in list(self.export.output_file_info.keys()) if f.endswith("tar.gz")
            ]
            import_response = self.imports_api.create(importer.pulp_href, {"path": filenames[0]})
        task_group = monitor_task_group(import_response.task_group)

        return task_group


class PulpImportTestCase(PulpImportTestBase):
    """
    Basic tests for PulpImporter and PulpImport.
    """

    @classmethod
    def setUpClass(cls):
        """Create class-wide variables."""
        cls.cfg = config.get_config()
        cls.client = api.Client(cls.cfg, api.json_handler)
        cls.core_client = CoreApiClient(configuration=cls.cfg.get_bindings_config())
        cls.rpm_client = gen_rpm_client()

        cls.repo_api = RepositoriesRpmApi(cls.rpm_client)
        cls.remote_api = RemotesRpmApi(cls.rpm_client)
        cls.exporter_api = ExportersPulpApi(cls.core_client)
        cls.exports_api = ExportersPulpExportsApi(cls.core_client)
        cls.importer_api = ImportersPulpApi(cls.core_client)
        cls.imports_api = ImportersPulpImportsApi(cls.core_client)

        cls.import_repos, cls.export_repos, cls.remotes = cls._setup_repositories()
        cls.exporter = cls._create_exporter()
        cls.export = cls._create_export()
        cls.chunked_export = cls._create_chunked_export()

    def test_import(self):
        """Test an import."""
        importer = self._create_importer()
        task_group = self._perform_import(importer)
        self.assertEqual(len(self.import_repos) + 1, task_group.completed)
        for repo in self.import_repos:
            repo = self.repo_api.read(repo.pulp_href)
            self.assertEqual(f"{repo.pulp_href}versions/1/", repo.latest_version_href)

    def test_double_import(self):
        """Test two imports of our export."""
        importer = self._create_importer()
        self._perform_import(importer)
        self._perform_import(importer)

        imports = self.imports_api.list(importer.pulp_href).results
        self.assertEqual(len(imports), 2)

        for repo in self.import_repos:
            repo = self.repo_api.read(repo.pulp_href)
            # still only one version as pulp won't create a new version if nothing changed
            self.assertEqual(f"{repo.pulp_href}versions/1/", repo.latest_version_href)


class DistributionTreePulpImportTestCase(PulpImportTestBase):
    """
    Tests for PulpImporter and PulpImport for repos with DistributionTrees.
    """

    @classmethod
    def setUpClass(cls):
        """Create class-wide variables."""
        cls.cfg = config.get_config()
        cls.client = api.Client(cls.cfg, api.json_handler)
        cls.core_client = CoreApiClient(configuration=cls.cfg.get_bindings_config())
        cls.rpm_client = gen_rpm_client()

        cls.repo_api = RepositoriesRpmApi(cls.rpm_client)
        cls.remote_api = RemotesRpmApi(cls.rpm_client)
        cls.exporter_api = ExportersPulpApi(cls.core_client)
        cls.exports_api = ExportersPulpExportsApi(cls.core_client)
        cls.importer_api = ImportersPulpApi(cls.core_client)
        cls.imports_api = ImportersPulpImportsApi(cls.core_client)
        cls.dist_tree_api = ContentDistributionTreesApi(cls.rpm_client)

        cls.import_repos, cls.export_repos, cls.remotes = cls._setup_repositories(
            RPM_KICKSTART_FIXTURE_URL
        )
        cls.exporter = cls._create_exporter()
        cls.export = cls._create_export()

    def test_import(self):
        """Test an import."""
        importer = self._create_importer()
        task_group = self._perform_import(importer)
        self.assertEqual(len(self.import_repos) + 1, task_group.completed)
        for repo in self.import_repos:
            repo = self.repo_api.read(repo.pulp_href)
            self.assertEqual(f"{repo.pulp_href}versions/1/", repo.latest_version_href)
            trees = self.dist_tree_api.list(repository_version=repo.latest_version_href)
            self.assertEqual(trees.count, 1)


class ParallelImportTestCase(PulpImportTestBase):
    """
    Tests for PulpImporter and PulpImport for repos with DistributionTrees.
    """

    @classmethod
    def setUpClass(cls):
        """Create class-wide variables."""
        cls.cfg = config.get_config()
        cls.client = api.Client(cls.cfg, api.json_handler)
        cls.core_client = CoreApiClient(configuration=cls.cfg.get_bindings_config())
        cls.rpm_client = gen_rpm_client()

        cls.repo_api = RepositoriesRpmApi(cls.rpm_client)
        cls.remote_api = RemotesRpmApi(cls.rpm_client)
        cls.exporter_api = ExportersPulpApi(cls.core_client)
        cls.exports_api = ExportersPulpExportsApi(cls.core_client)
        cls.importer_api = ImportersPulpApi(cls.core_client)
        cls.imports_api = ImportersPulpImportsApi(cls.core_client)
        cls.dist_tree_api = ContentDistributionTreesApi(cls.rpm_client)

        cls.import_repos, cls.export_repos, cls.remotes = cls._setup_repositories(
            RPM_UNSIGNED_FIXTURE_URL
        )
        cls.exporter = cls._create_exporter()
        cls.export = cls._create_export()

    @classmethod
    def tearDownClass(cls):
        """No need for class to clean up, we've already done it."""
        pass

    def tearDown(self):
        """Remaining clean up repos/export already gone by now."""
        for repo in self.import_repos:
            self.repo_api.delete(repo.pulp_href)
        delete_orphans()

    def _post_export_cleanup(self):
        """
        Remove the exported repos and their content, so we're importing into a 'clean' db.
        """
        self._delete_exporter(delete_file=False)
        for remote in self.remotes:
            self.remote_api.delete(remote.pulp_href)
        for repo in self.export_repos:
            self.repo_api.delete(repo.pulp_href)
        delete_orphans()

    def test_clean_import(self):
        """Test an import into an empty instance."""
        # By the time we get here, setUpClass() has created repos and an export for us
        # Find the export file, create the importer, delete repos/export, and then let
        # the import happen
        filenames = [f for f in list(self.export.output_file_info.keys()) if f.endswith("tar.gz")]
        importer = self._create_importer()
        self._post_export_cleanup()

        # At this point we should be importing into a content-free-zone
        import_response = self.imports_api.create(importer.pulp_href, {"path": filenames[0]})
        task_group = monitor_task_group(import_response.task_group)
        self.assertEqual(len(self.import_repos) + 1, task_group.completed)
        for repo in self.import_repos:
            repo = self.repo_api.read(repo.pulp_href)
            self.assertEqual(f"{repo.pulp_href}versions/1/", repo.latest_version_href)



"""Constants for Pulp RPM plugin tests."""

from urllib.parse import urljoin
from pulp_smash import config
from pulp_smash.pulp3.constants import (
    BASE_CONTENT_PATH,
    BASE_DISTRIBUTION_PATH,
    BASE_REPO_PATH,
    BASE_PATH,
    BASE_PUBLICATION_PATH,
    BASE_REMOTE_PATH,
)

RPM_COPY_PATH = urljoin(BASE_PATH, "rpm/copy/")
"""The URL used for copying RPM content between repos."""

PULP_FIXTURES_BASE_URL = config.get_config().get_fixtures_url()

DOWNLOAD_POLICIES = ["immediate", "on_demand", "streamed"]

DRPM_UNSIGNED_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "drpm-unsigned/")
"""The URL to a repository with unsigned DRPM packages."""

RPM_PACKAGE_CONTENT_NAME = "rpm.package"

RPM_PACKAGECATEGORY_CONTENT_NAME = "rpm.packagecategory"

RPM_PACKAGEENVIRONMENT_CONTENT_NAME = "rpm.packageenvironment"

RPM_PACKAGEGROUP_CONTENT_NAME = "rpm.packagegroup"

RPM_PACKAGELANGPACKS_CONTENT_NAME = "rpm.packagelangpacks"

RPM_ADVISORY_CONTENT_NAME = "rpm.advisory"

RPM_ALT_LAYOUT_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-alt-layout/")
"""The URL to a signed RPM repository. See :data:`RPM_SIGNED_FIXTURE_URL`."""

RPM_CONTENT_PATH = urljoin(BASE_CONTENT_PATH, "rpm/packages/")
"""The location of RPM packages on the content endpoint."""

RPM_NAMESPACES = {
    "metadata/common": "http://linux.duke.edu/metadata/common",
    "metadata/filelists": "http://linux.duke.edu/metadata/filelists",
    "metadata/other": "http://linux.duke.edu/metadata/other",
    "metadata/repo": "http://linux.duke.edu/metadata/repo",
    "metadata/rpm": "http://linux.duke.edu/metadata/rpm",
}
"""Namespaces used by XML-based RPM metadata.

Many of the XML files generated by the ``createrepo`` utility make use of these
namespaces. Some of the files that use these namespaces are listed below:

metadata/common
    Used by ``repodata/primary.xml``.

metadata/filelists
    Used by ``repodata/filelists.xml``.

metadata/other
    Used by ``repodata/other.xml``.

metadata/repo
    Used by ``repodata/repomd.xml``.

metadata/rpm
    Used by ``repodata/repomd.xml``.
"""

RPM_DISTRIBUTION_PATH = urljoin(BASE_DISTRIBUTION_PATH, "rpm/rpm/")

RPM_REMOTE_PATH = urljoin(BASE_REMOTE_PATH, "rpm/rpm/")

RPM_REPO_PATH = urljoin(BASE_REPO_PATH, "rpm/rpm/")

RPM_PUBLICATION_PATH = urljoin(BASE_PUBLICATION_PATH, "rpm/rpm/")

RPM_SHA512_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-with-sha-512/")
"""The URL to an RPM repository with sha512 checksum."""

RPM_SIGNED_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-signed/")
"""The URL to a repository with signed RPM packages."""

RPM_COMPLEX_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-complex-pkg/")

RPM_SINGLE_REQUEST_UPLOAD = urljoin(BASE_PATH, "content/rpm/packages/")

RPM_UNSIGNED_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-unsigned/")
"""The URL to a repository with unsigned RPM packages."""

RPM_INVALID_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-missing-filelists/")

RPM_MIRROR_LIST_GOOD_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-mirrorlist-good")
RPM_MIRROR_LIST_BAD_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-mirrorlist-bad")

RPM_SHA_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-with-sha")

RPM_PACKAGE_COUNT = 35
"""The number of packages available at
:data:`RPM_SIGNED_FIXTURE_URL` and :data:`RPM_UNSIGNED_FIXTURE_URL`
"""

RPM_PACKAGECATEGORY_COUNT = 1
"""The number of packagecategories."""

RPM_PACKAGEGROUP_COUNT = 2
"""The number of packagegroups."""

RPM_PACKAGELANGPACKS_COUNT = 1
"""The number of packagelangpacks."""

RPM_ADVISORY_COUNT = 4
"""The number of updated record units."""

RPM_FIXTURE_SUMMARY = {
    RPM_PACKAGE_CONTENT_NAME: RPM_PACKAGE_COUNT,
    RPM_ADVISORY_CONTENT_NAME: RPM_ADVISORY_COUNT,
    RPM_PACKAGECATEGORY_CONTENT_NAME: RPM_PACKAGECATEGORY_COUNT,
    RPM_PACKAGEGROUP_CONTENT_NAME: RPM_PACKAGEGROUP_COUNT,
    RPM_PACKAGELANGPACKS_CONTENT_NAME: RPM_PACKAGELANGPACKS_COUNT,
}
"""The breakdown of how many of each type of content unit are present in the
standard repositories, i.e. :data:`RPM_SIGNED_FIXTURE_URL` and
:data:`RPM_UNSIGNED_FIXTURE_URL`.  This matches the format output by the
"content_summary" field on "../repositories/../versions/../".
"""

FEDORA_MIRRORLIST_BASE = "https://mirrors.fedoraproject.org/mirrorlist"
FEDORA_MIRRORLIST_PARAMS = "?repo=epel-modular-8&arch=x86_64&infra=stock&content=centos"
RPM_EPEL_MIRROR_URL = FEDORA_MIRRORLIST_BASE + FEDORA_MIRRORLIST_PARAMS
"""The URL to retrieve a mirrorlist for the EPEL-8-MODULAR repo.

.. NOTE:: This repository is not generated by pulp-fixtures.
"""

RPM_LONG_UPDATEINFO_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-long-updateinfo/")
"""The URL to RPM with a long updateinfo.xml."""

RPM_MODULAR_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-modular/")
"""The URL to a modular RPM repository."""

RPM_MODULAR_PACKAGE_COUNT = 25
"""The number of packages present on `RPM_MODULAR_FIXTURE_URL`."""

RPM_MODULAR_MODULES_COUNT = 11
"""The number of modules present on `RPM_MODULAR_FIXTURE_URL`."""

RPM_MODULAR_DEFAULTS_COUNT = 4
"""The number of modules-default present on `RPM_MODULAR_FIXTURE_URL`."""

RPM_MODULAR_ADVISORY_COUNT = 1
"""The number of advisories present on `RPM_MODULAR_FIXTURE_URL`."""

RPM_MODULAR_MODULES_CONTENT_NAME = "rpm.modulemd"

RPM_MODULAR_DEFAULTS_CONTENT_NAME = "rpm.modulemd_defaults"

RPM_MODULES_OBSOLETE_CONTENT_NAME = "rpm.modulemd_obsolete"

RPM_MODULES_OBSOLETE_COUNT = 6
"""The number of modules-default present on `RPM_MODULAR_FIXTURE_URL`."""

RPM_MODULAR_FIXTURE_SUMMARY = {
    RPM_PACKAGE_CONTENT_NAME: RPM_MODULAR_PACKAGE_COUNT,
    RPM_MODULAR_MODULES_CONTENT_NAME: RPM_MODULAR_MODULES_COUNT,
    RPM_MODULAR_DEFAULTS_CONTENT_NAME: RPM_MODULAR_DEFAULTS_COUNT,
    RPM_MODULES_OBSOLETE_CONTENT_NAME: RPM_MODULES_OBSOLETE_COUNT,
    RPM_ADVISORY_CONTENT_NAME: RPM_MODULAR_ADVISORY_COUNT,
}

RPM_MODULES_STATIC_CONTEXT_FIXTURE_URL = urljoin(
    PULP_FIXTURES_BASE_URL, "rpm-modules-static-context/"
)
"""The URL to a modular RPM repository that uses the static_context field."""

RPM_MODULAR_STATIC_FIXTURE_SUMMARY = {
    RPM_PACKAGE_CONTENT_NAME: RPM_MODULAR_PACKAGE_COUNT,
    RPM_MODULAR_MODULES_CONTENT_NAME: RPM_MODULAR_MODULES_COUNT,
    RPM_MODULAR_DEFAULTS_CONTENT_NAME: RPM_MODULAR_DEFAULTS_COUNT,
    RPM_MODULES_OBSOLETE_CONTENT_NAME: RPM_MODULES_OBSOLETE_COUNT,
    RPM_ADVISORY_CONTENT_NAME: RPM_MODULAR_ADVISORY_COUNT,
}

"""The breakdown of how many of each type of content unit are present in the
i.e. :data:`RPM_MODULAR_FIXTURE_URL`."""

RPM_PACKAGE_DATA = {
    "name": "kangaroo",
    "epoch": "0",
    "version": "0.3",
    "release": "1",
    "arch": "noarch",
    "description": "A modular RPM fixture for testing Pulp.",
    "summary": "hop like a kangaroo in Australia",
    "rpm_license": "Public Domain",
    "rpm_group": "Unspecified",
    "rpm_vendor": "",
    # TODO: Complete this information once we figure out how to serialize
    # everything nicely
}
"""The metadata for one RPM package."""


RPM_COMPLEX_PACKAGE_DATA = {
    "arch": "x86_64",
    "artifact": None,
    "changelogs": [
        [
            "Lucille Bluth <lucille@bluthcompany.com> - 1.1.1-1",
            1617192000,
            "- It's a banana, Michael. How much could it cost, $10?",
        ],
        [
            "Job Bluth <job@alliance-of-magicians.com> - 2.2.2-2",
            1619352000,
            "- I've made a huge mistake",
        ],
        [
            "George Bluth <george@federalprison.gov> - 3.3.3-3",
            1623672000,
            "- There’s always money in the banana stand",
        ],
    ],
    "checksum_type": "sha256",
    "conflicts": [["foxnetwork", "GT", "0", "5555", None, False]],
    "description": "Complex package",
    "enhances": [["(bananas or magic)", None, None, None, None, False]],
    "epoch": "1",
    "files": [
        [None, "/etc/complex/", "pkg.cfg"],
        [None, "/usr/bin/", "complex_a"],
        ["dir", "/usr/share/doc/", "complex-package"],
        [None, "/usr/share/doc/complex-package/", "README"],
        ["dir", "/var/lib/", "complex"],
        ["ghost", "/var/log/", "complex.log"],
    ],
    "is_modular": False,
    "location_base": "",
    "location_href": "complex-package-2.3.4-5.el8.x86_64.rpm",
    "md5": None,
    "name": "complex-package",
    "obsoletes": [
        ["bluemangroup", "LT", "0", "32.1", "0", False],
        ["cornballer", "LT", "0", "444", None, False],
    ],
    "pkg_id": "bbb7b0e9350a0f75b923bdd0ef4f9af39765c668a3e70bfd3486ea9f0f618aaf",
    "provides": [
        ["/usr/bin/ls", None, None, None, None, False],
        ["complex-package", "EQ", "1", "2.3.4", "5.el8", False],
        ["complex-package(x86-64)", "EQ", "1", "2.3.4", "5.el8", False],
        ["laughter", "EQ", "0", "33", None, False],
        ["narration(ronhoward)", None, None, None, None, False],
    ],
    "recommends": [
        ["((hiding and attic) if light-treason)", None, None, None, None, False],
        ["GeneParmesan(PI)", None, None, None, None, False],
        ["yacht", "GT", "9", "11.0", "0", False],
    ],
    "release": "5.el8",
    "requires": [
        ["/usr/bin/bash", None, None, None, None, False],
        ["/usr/sbin/useradd", None, None, None, None, True],
        ["arson", "GE", "0", "1.0.0", "1", False],
        ["fur", "LE", "0", "2", None, False],
        ["staircar", "LE", "0", "99.1", "3", False],
    ],
    "rpm_buildhost": "localhost",
    "rpm_group": "Development/Tools",
    "rpm_header_end": 8413,
    "rpm_header_start": 4504,
    "rpm_license": "MPLv2",
    "rpm_packager": "Michael Bluth",
    "rpm_sourcerpm": "complex-package-2.3.4-5.el8.src.rpm",
    "rpm_vendor": "Bluth Company",
    "sha1": None,
    "sha224": None,
    "sha256": None,
    "sha384": None,
    "sha512": None,
    "size_archive": 932,
    "size_installed": 117,
    "size_package": 8680,
    "suggests": [
        ["(bobloblaw >= 1.1 if maritimelaw else anyone < 0.5.1-2)", None, None, None, None, False],
        ["(dove and return)", None, None, None, None, False],
        ["(job or money > 9000)", None, None, None, None, False],
    ],
    "summary": "A package for exercising many different features of RPM metadata",
    "supplements": [
        ["((hiding and illusion) unless alliance-of-magicians)", None, None, None, None, False],
        ["comedy", "EQ", "0", "11.1", "4", False],
    ],
    "time_build": 1627052743,
    "time_file": 1627056000,
    "url": "http://bobloblaw.com",
    "version": "2.3.4",
}


RPM_PACKAGE_DATA2 = {
    "name": "duck",
    "epoch": "0",
    "version": "0.7",
    "release": "1",
    "arch": "noarch",
    "description": "A dummy package of duck",
    "summary": "A dummy package of duck",
    "rpm_license": "GPLv2",
    "rpm_group": "Internet/Applications",
    "rpm_vendor": "",
    # TODO: Complete this information once we figure out how to serialize
    # everything nicely
}
"""The metadata for one RPM package."""

RPM_PACKAGE_NAME = "{}".format(RPM_PACKAGE_DATA["name"])
"""The name of one RPM package."""

RPM_PACKAGE_FILENAME = "{}-{}-{}.{}.rpm".format(
    RPM_PACKAGE_DATA["name"],
    RPM_PACKAGE_DATA["version"],
    RPM_PACKAGE_DATA["release"],
    RPM_PACKAGE_DATA["arch"],
)
"""The filename of one RPM package."""

RPM_PACKAGE_FILENAME2 = "{}-{}-{}.{}.rpm".format(
    RPM_PACKAGE_DATA2["name"],
    RPM_PACKAGE_DATA2["version"],
    RPM_PACKAGE_DATA2["release"],
    RPM_PACKAGE_DATA2["arch"],
)
"""The filename of one RPM package."""

RPM_REFERENCES_UPDATEINFO_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-references-updateinfo/")
"""The URL to a repository with ``updateinfo.xml`` containing references.

This repository includes advisory with reference element (0, 1 or 2 references)
and without it.
"""

RPM_RICH_WEAK_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-richnweak-deps/")
"""The URL to an RPM repository with weak and rich dependencies."""

RPM_SIGNED_URL = urljoin(RPM_SIGNED_FIXTURE_URL, RPM_PACKAGE_FILENAME)
"""The path to a single signed RPM package."""

RPM_SIGNED_URL2 = urljoin(RPM_SIGNED_FIXTURE_URL, RPM_PACKAGE_FILENAME2)
"""The path to a single signed RPM package."""

RPM_UNSIGNED_URL = urljoin(RPM_UNSIGNED_FIXTURE_URL, RPM_PACKAGE_FILENAME)
"""The path to a single unsigned RPM package."""

RPM_UPDATED_UPDATEINFO_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-updated-updateinfo/")
"""The URL to a repository containing UpdateRecords (Advisory) with the same IDs
as the ones in the standard repositories, but with different metadata.

Note: This repository uses unsigned RPMs.
"""

RPM_ADVISORY_UPDATED_VERSION_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-updated-updateversion")
"""The URL to a repository containing Advisories with same ID as the ones in the standard
unsigned rpm repository, but with updated Advisory version.
"""

RPM_ADVISORY_DIFFERENT_PKGLIST_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-advisory-diffpkgs")
"""The URL to a repository containing Advisories with same ID and version as the ones in the
standard unsigned rpm repository, but with different pkglist.
"""

RPM_ADVISORY_DIFFERENT_REPO_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-advisory-diff-repo")
"""The URL to a repository containing Advisories with same ID and version as the ones in the
standard unsigned rpm repository, but with different update_date and packages intersection.
"""

RPM_ADVISORY_INCOMPLETE_PKG_LIST_URL = urljoin(
    PULP_FIXTURES_BASE_URL, "rpm-advisory-incomplete-package-list"
)
"""The URL to a repository containing Advisories with same update_date and version as the ones
in the standard unsigned rpm repository, but pkglist intersection is non-empty and not equal
to either pkglist.
"""

RPM_ADVISORY_NO_UPDATED_DATE = urljoin(PULP_FIXTURES_BASE_URL, "rpm-advisory-no-update-date")
"""The URL to a repository containing Advisories with same id and version as the ones
in the standard unsigned rpm repository, but no updated_date.
"""

RPM_ADVISORY_TEST_ID = "RHEA-2012:0056"
RPM_ADVISORY_TEST_ID_NEW = "RHEA-2012:0058"
"""The ID of an UpdateRecord (advisory/erratum).

The package contained on this advisory is defined by
:data:`RPM_UPDATERECORD_RPM_NAME` and the advisory is present in the standard
repositories, i.e. :data:`RPM_SIGNED_FIXTURE_URL` and
:data:`RPM_UNSIGNED_FIXTURE_URL`.
"""
RPM_ADVISORY_TEST_REMOVE_COUNT = 3
RPM_ADVISORY_TEST_ADDED_COUNT = 6


RPM_REPO_METADATA_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-repo-metadata/")
"""The URL to RPM repository with custom repository metadata."""

RPM_UPDATERECORD_RPM_NAME = "gorilla"
"""The name of the RPM named by :data:`RPM_UPDATERECORD_ID`."""

RPM_WITH_NON_ASCII_NAME = "rpm-with-non-ascii"

RPM_WITH_NON_ASCII_URL = urljoin(
    PULP_FIXTURES_BASE_URL,
    "rpm-with-non-ascii/{}-1-1.fc25.noarch.rpm".format(RPM_WITH_NON_ASCII_NAME),
)
"""The URL to an RPM with non-ascii metadata in its header."""

RPM_WITH_NON_UTF_8_NAME = "rpm-with-non-utf-8"

RPM_WITH_NON_UTF_8_URL = urljoin(
    PULP_FIXTURES_BASE_URL,
    "rpm-with-non-utf-8/{}-1-1.fc25.noarch.rpm".format(RPM_WITH_NON_UTF_8_NAME),
)
"""The URL to an RPM with non-UTF-8 metadata in its header."""

SRPM_UNSIGNED_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "srpm-unsigned/")
"""The URL to a repository with unsigned SRPM packages."""

SRPM_UNSIGNED_FIXTURE_ADVISORY_COUNT = 2
"""Count of advisories in the repository."""

SRPM_UNSIGNED_FIXTURE_PACKAGE_COUNT = 3
"""Count of packages in the repository."""

UPDATERECORD_CONTENT_PATH = urljoin(BASE_CONTENT_PATH, "rpm/advisories/")
"""The location of RPM UpdateRecords on the content endpoint."""

KICKSTART_CONTENT_PATH = urljoin(BASE_CONTENT_PATH, "rpm/distribution_trees/")
"""The location of RPM Distribution Trees on the content endpoint."""

RPM_KICKSTART_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-distribution-tree/")
RPM_KICKSTART_ONLY_META_FIXTURE_URL = urljoin(
    PULP_FIXTURES_BASE_URL, "rpm-distribution-tree-metadata-only/"
)
RPM_DISTRIBUTION_TREE_CHANGED_ADDON_URL = urljoin(
    PULP_FIXTURES_BASE_URL, "rpm-distribution-tree-changed-addon/"
)
RPM_DISTRIBUTION_TREE_CHANGED_MAIN_URL = urljoin(
    PULP_FIXTURES_BASE_URL, "rpm-distribution-tree-changed-main/"
)
RPM_DISTRIBUTION_TREE_CHANGED_VARIANT_URL = urljoin(
    PULP_FIXTURES_BASE_URL, "rpm-distribution-tree-changed-variant/"
)

RPM_KICKSTART_CONTENT_NAME = "rpm.distribution_tree"

RPM_KICKSTART_COUNT = 1

RPM_KICKSTART_FIXTURE_SUMMARY = {
    RPM_KICKSTART_CONTENT_NAME: RPM_KICKSTART_COUNT,
    RPM_PACKAGE_CONTENT_NAME: 1,
    RPM_PACKAGECATEGORY_CONTENT_NAME: 1,
    RPM_PACKAGEENVIRONMENT_CONTENT_NAME: 1,
    RPM_PACKAGEGROUP_CONTENT_NAME: 1,
    RPM_PACKAGELANGPACKS_CONTENT_NAME: 1,
}

RPM_KICKSTART_REPOSITORY_ROOT_CONTENT = [
    ".treeinfo",
    "Dolphin/",
    "External/",
    "Land/",
    "LiveOS/",
    "Packages/",
    "Whale/",
    "images/",
]

RPM_KICKSTART_DATA = {
    "addons": [
        {
            "addon_id": "Whale",
            "name": "Whale addon",
            "packages": "Whale/Packages",
            "type": "addon",
            "uid": "Sea-Whale",
        },
        {
            "addon_id": "Dolphin",
            "name": "Dolphin addon",
            "packages": "Dolphin/Packages",
            "type": "addon",
            "uid": "Sea-Dolphin",
        },
    ],
    "arch": "x86_64",
    "base_product_name": None,
    "base_product_short": None,
    "base_product_version": None,
    "build_timestamp": 1539194952.0,
    "checksums": [
        {
            "checksum": "sha256:91052b444e73f3eebdb93d1fb1506597e96c92d8de9c1e3c3f36b07a57d0a18f",
            "path": "images/pxeboot/vmlinuz",
        },
        {
            "checksum": "sha256:30e14955ebf1352266dc2ff8067e68104607e750abb9d3b36582b8af909fcb58",
            "path": "images/pxeboot/upgrade.img",
        },
        {
            "checksum": "sha256:5647f05ec18958947d32874eeb788fa396a05d0bab7c1b71f112ceb7e9b31eee",
            "path": "images/pxeboot/initrd.img",
        },
        {
            "checksum": "sha256:54260288eda2bd151c71358c5f2c472654d54420dbf3f2c1aafcb933a42a3074",
            "path": "fixtures/rpm-distribution-tree/addons/whale/repodata/repomd.xml",
        },
        {
            "checksum": "sha256:b6ac17b542fc6188eafadb932d9635209e1498c1ee22ba2490f531fdbd4c49a3",
            "path": "fixtures/rpm-distribution-tree/addons/dolphin/repodata/repomd.xml",
        },
        {
            "checksum": "sha256:30e14955ebf1352266dc2ff8067e68104607e750abb9d3b36582b8af909fcb58",
            "path": "LiveOS/squashfs.img",
        },
    ],
    "discnum": 1,
    "header_version": "1.2",
    "images": [
        {
            "artifact": None,
            "name": "upgrade",
            "path": "images/pxeboot/upgrade.img",
            "platforms": "x86_64, xen",
        },
        {
            "artifact": None,
            "name": "kernel",
            "path": "images/pxeboot/vmlinuz",
            "platforms": "x86_64, xen",
        },
        {
            "artifact": None,
            "name": "initrd",
            "path": "images/pxeboot/initrd.img",
            "platforms": "x86_64, xen",
        },
    ],
    "instimage": None,
    "mainimage": "LiveOS/squashfs.img",
    "release_is_layered": False,
    "release_name": "Red Hat Enterprise Linux",
    "release_short": "RHEL",
    "release_version": "7.6",
    "totaldiscs": 1,
    "variants": [
        {
            "debug_packages": None,
            "debug_repository": None,
            "identity": None,
            "name": "Sea",
            "packages": "Packages",
            "source_packages": None,
            "source_repository": None,
            "type": "variant",
            "uid": "Sea",
            "variant_id": "Sea",
        },
        {
            "debug_packages": None,
            "debug_repository": None,
            "identity": None,
            "name": "Land",
            "packages": "Land/Packages",
            "source_packages": None,
            "source_repository": None,
            "type": "variant",
            "uid": "Land",
            "variant_id": "Land",
        },
        {
            "debug_packages": None,
            "debug_repository": None,
            "identity": None,
            "name": "External",
            "packages": "External/Packages",
            "source_packages": None,
            "source_repository": None,
            "type": "variant",
            "uid": "External",
            "variant_id": "External",
        },
    ],
}

PULP_FIXTURES_COMMON_URL = "https://github.com/pulp/pulp-fixtures/raw/master/common/"
PUBLIC_GPG_KEY_URL = urljoin(PULP_FIXTURES_COMMON_URL, "GPG-KEY-pulp-qe")
PRIVATE_GPG_KEY_URL = urljoin(PULP_FIXTURES_COMMON_URL, "GPG-PRIVATE-KEY-pulp-qe")

RPM_CUSTOM_REPO_METADATA_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-repo-metadata/")
RPM_CUSTOM_REPO_METADATA_CHANGED_FIXTURE_URL = urljoin(
    PULP_FIXTURES_BASE_URL, "rpm-repo-metadata-changed/"
)

RPM_MD5_REPO_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-with-md5/")

RPM_DIFF_NAME_SAME_CONTENT_URL = urljoin(PULP_FIXTURES_BASE_URL, "diff-name-same-content/")

RPM_ONLY_METADATA_REPO_URL = urljoin(PULP_FIXTURES_BASE_URL, "rpm-unsigned-meta-only")

AMAZON_MIRROR = "http://amazonlinux.us-east-1.amazonaws.com/2/core/latest/x86_64/mirror.list"
CENTOS7_URL = "http://mirror.centos.org/centos-7/7/os/x86_64/"
CENTOS7_OPSTOOLS_URL = "http://mirror.centos.org/centos/7/opstools/x86_64/"
CENTOS8_STREAM_BASEOS_URL = "http://mirror.centos.org/centos/8-stream/BaseOS/x86_64/os/"
CENTOS8_STREAM_APPSTREAM_URL = "http://mirror.centos.org/centos/8-stream/AppStream/x86_64/os/"
CENTOS9_STREAM_BASEOS_URL = "http://mirror.stream.centos.org/9-stream/BaseOS/x86_64/os/"
CENTOS9_STREAM_APPSTREAM_URL = "http://mirror.stream.centos.org/9-stream/AppStream/x86_64/os/"
EPEL7_URL = "https://dl.fedoraproject.org/pub/epel/7/x86_64/"
EPEL8_MIRRORLIST_URL = "https://mirrors.fedoraproject.org/mirrorlist?repo=epel-8&arch=x86_64"
F36_KICKSTART_URL = "https://dl.fedoraproject.org/pub/fedora/linux/releases/36/Server/x86_64/os/"
RHEL6_KICKSTART_CDN_URL = "https://cdn.redhat.com/content/dist/rhel/server/6/6.10/x86_64/kickstart/"
RHEL8_BASEOS_CDN_URL = "https://cdn.redhat.com/content/dist/rhel8/8/x86_64/baseos/os/"
RHEL8_APPSTREAM_CDN_URL = "https://cdn.redhat.com/content/dist/rhel8/8/x86_64/appstream/os/"
RHEL9_BASEOS_CDN_URL = "https://cdn.redhat.com/content/dist/rhel9/9/x86_64/baseos/os/"
RHEL9_APPSTREAM_CDN_URL = "https://cdn.redhat.com/content/dist/rhel9/9/x86_64/appstream/os/"

# TODO: static test fixtures for these
REPO_WITH_XML_BASE_URL = "https://harbottle.gitlab.io/harbottle-main/8/x86_64/"
REPO_WITH_EXTERNAL_LOCATION_HREF_URL = (
    # This URL no longer works for this purpose
    "https://packages.rundeck.com/pagerduty/rundeck/rpm_any/rpm_any/x86_64/"
)

PULP_TYPE_ADVISORY = "rpm.advisory"
PULP_TYPE_DISTRIBUTION_TREE = "rpm.distribution_tree"
PULP_TYPE_PACKAGE = "rpm.package"
PULP_TYPE_PACKAGE_CATEGORY = "rpm.packagecategory"
PULP_TYPE_PACKAGE_GROUP = "rpm.packagegroup"
PULP_TYPE_REPOMETADATA = "rpm.repo_metadata_file"
PULP_TYPE_MODULEMD = "rpm.modulemd"
PULP_TYPE_MODULEMD_DEFAULTS = "rpm.modulemd_defaults"

PACKAGES_DIRECTORY = "Packages"

"""The URL used for uploading comps.xml files."""
COMPS_API = urljoin(BASE_PATH, "rpm/comps/")
SMALL_COMPS_XML = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE comps PUBLIC "-//Red Hat, Inc.//DTD Comps info//EN" "comps.dtd">
<comps>
  <group>
   <id>birds</id>
   <default>true</default>
   <uservisible>true</uservisible>
   <display_order>1024</display_order>
   <name>birds</name>
   <description></description>
    <packagelist>
      <packagereq type="mandatory">cockateel</packagereq>
      <packagereq type="mandatory">duck</packagereq>
      <packagereq type="mandatory">penguin</packagereq>
      <packagereq type="mandatory">stork</packagereq>
    </packagelist>
  </group>
  <group>
   <id>mammals</id>
   <default>true</default>
   <uservisible>true</uservisible>
   <display_order>1024</display_order>
   <name>mammals</name>
   <description></description>
    <packagelist>
      <packagereq type="mandatory">bear</packagereq>
      <packagereq type="mandatory">camel</packagereq>
      <packagereq type="mandatory">cat</packagereq>
      <packagereq type="mandatory">cheetah</packagereq>
      <packagereq type="mandatory">chimpanzee</packagereq>
      <packagereq type="mandatory">cow</packagereq>
      <packagereq type="mandatory">dog</packagereq>
      <packagereq type="mandatory">dolphin</packagereq>
      <packagereq type="mandatory">elephant</packagereq>
      <packagereq type="mandatory">fox</packagereq>
      <packagereq type="mandatory">giraffe</packagereq>
      <packagereq type="mandatory">gorilla</packagereq>
      <packagereq type="mandatory">horse</packagereq>
      <packagereq type="mandatory">kangaroo</packagereq>
      <packagereq type="mandatory">lion</packagereq>
      <packagereq type="mandatory">mouse</packagereq>
      <packagereq type="mandatory">squirrel</packagereq>
      <packagereq type="mandatory">tiger</packagereq>
      <packagereq type="mandatory">walrus</packagereq>
      <packagereq type="mandatory">whale</packagereq>
      <packagereq type="mandatory">wolf</packagereq>
      <packagereq type="mandatory">zebra</packagereq>
    </packagelist>
  </group>
  <category>
   <id>all</id>
   <display_order>99</display_order>
   <name>all</name>
   <description></description>
    <grouplist>
     <groupid>mammals</groupid>
     <groupid>birds</groupid>
    </grouplist>
  </category>
  <langpacks>
    <match install="gorilla-%s" name="gorilla-en"/>
    <match install="zebra-%s" name="zebra-docs"/>
  </langpacks>
</comps>"""
SMALL_GROUPS = 2
SMALL_CATEGORY = 1
SMALL_LANGPACK = 1
SMALL_ENVIRONMENTS = 0

BIG_COMPS_XML = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE comps
  PUBLIC '-//Red Hat, Inc.//DTD Comps info//EN'
  'comps.dtd'>
<comps>
  <group>
    <id>additional-devel</id>
    <name>Additional Development</name>
    <name xml:lang="as">অতিৰিক্ত উন্নয়ন</name>
    <name xml:lang="bn">অতিরিক্ত ডিভেলপমেন্ট</name>
    <name xml:lang="cs">Doplňky pro vývoj</name>
    <name xml:lang="de">Zusätzliche Entwicklung</name>
    <name xml:lang="de_CH">Zusätzliche Entwicklung</name>
    <name xml:lang="es">Desarrollo adicional</name>
    <name xml:lang="fr">Développement supplémentaire</name>
    <name xml:lang="gu">વધારાનાં વિકાસ</name>
    <name xml:lang="hi">अतिरिक्त विकास</name>
    <name xml:lang="ia">Disveloppamento additional</name>
    <name xml:lang="it">Sviluppo aggiuntivo</name>
    <name xml:lang="ja">その他の開発</name>
    <name xml:lang="kn">ಹೆಚ್ಚುವರಿ ವಿಕಸನೆ</name>
    <name xml:lang="ko">추가 개발</name>
    <name xml:lang="ml">അധികമായ ഡവലപ്പ്മെന്റ്</name>
    <name xml:lang="mr">अगाऊ डेव्हलपमेंट</name>
    <name xml:lang="or">ଅତିରିକ୍ତ ସର୍ଭର ବିକାଶ</name>
    <name xml:lang="pa">ਵਧੀਕ ਵਿਕਾਸ</name>
    <name xml:lang="pl">Dodatkowe programowanie</name>
    <name xml:lang="pt">Desenvolvimento adicional</name>
    <name xml:lang="pt_BR">Desenvolvimento adicional</name>
    <name xml:lang="ru">Дополнительные средства разработки</name>
    <name xml:lang="sv">Ytterligare utveckling</name>
    <name xml:lang="ta">கூடுதல் உருவாக்கம்</name>
    <name xml:lang="te">అదనపు అభివృద్ది</name>
    <name xml:lang="uk">Додаткові інструменти розробки</name>
    <name xml:lang="zh">額外開發</name>
    <name xml:lang="zh_CN">附加开发</name>
    <name xml:lang="zh_TW">額外開發</name>
    <description>Additional development headers and libraries for building open-source</description>
    <description xml:lang="cs">Doplňkové knihovny a hlavičkové soubory pro sestavování</description>
    <description xml:lang="de">Zusätzliche Entwicklungs-Header und -Bibliotheken zur E</description>
    <description xml:lang="de_CH">Zusätzliche Entwicklungs-Header und -Bibliotheken zu</description>
    <description xml:lang="es">Encabezados adicionales y bibliotecas para compilar apl</description>
    <description xml:lang="fr">En-têtes et bibliothèques de développement supplémentai</description>
    <description xml:lang="it">Librerie e intestazioni per uno sviluppo aggiuntivo per</description>
    <description xml:lang="ko">오픈 소스 애플리케이션 구축을 위한 기타 개발 헤더 및 라이브러리입니다.</description>
    <description xml:lang="pl">Dodatkowe nagłówki i biblioteki do rozwijania aplikacji</description>
    <description xml:lang="pt">Cabeçalhos de desenvolvimento adiiconal e bibliotecas p</description>
    <description xml:lang="pt_BR">Cabeçalhos de desenvolvimento adiiconal e biblioteca</description>
    <description xml:lang="ru">Дополнительные библиотеки и файлы заголовков для разраб</description>
    <description xml:lang="uk">Додаткові пакунки з заголовками та бібліотеками для роз</description>
    <description xml:lang="zh">用來建置開放式原始碼應用程式的應用程式開發表頭與函式庫。</description>
    <description xml:lang="zh_CN">用于构建开源应用程序的附加开发标头及程序可。</description>
    <description xml:lang="zh_TW">用來建置開放式原始碼應用程式的應用程式開發表頭與函式庫。</description>
    <default>true</default>
    <uservisible>false</uservisible>
    <packagelist>
      <packagereq type="mandatory">atlas-devel</packagereq>
      <packagereq type="mandatory">audit-libs-devel</packagereq>
      <packagereq type="mandatory">bzip2-devel</packagereq>
      <packagereq type="mandatory">c-ares-devel</packagereq>
      <packagereq type="mandatory">cyrus-sasl-devel</packagereq>
      <packagereq type="mandatory">e2fsprogs-devel</packagereq>
      <packagereq type="mandatory">elfutils-devel</packagereq>
      <packagereq type="mandatory">elfutils-libelf-devel</packagereq>
      <packagereq type="mandatory">expat-devel</packagereq>
      <packagereq type="mandatory">fuse3-devel</packagereq>
      <packagereq type="mandatory">gmp-devel</packagereq>
      <packagereq type="mandatory">hmaccalc</packagereq>
      <packagereq type="mandatory">iptables-devel</packagereq>
      <packagereq type="mandatory">libacl-devel</packagereq>
      <packagereq type="mandatory">libaio-devel</packagereq>
      <packagereq type="mandatory">libattr-devel</packagereq>
      <packagereq type="mandatory">libblkid-devel</packagereq>
      <packagereq type="mandatory">libcap-devel</packagereq>
      <packagereq type="mandatory">libcap-ng-devel</packagereq>
      <packagereq type="mandatory">libcurl-devel</packagereq>
      <packagereq type="mandatory">libffi-devel</packagereq>
      <packagereq type="mandatory">libgcrypt-devel</packagereq>
      <packagereq type="mandatory">libhugetlbfs-devel</packagereq>
      <packagereq type="mandatory">libnl3-devel</packagereq>
      <packagereq type="mandatory">libselinux-devel</packagereq>
      <packagereq type="mandatory">libusbx-devel</packagereq>
      <packagereq type="mandatory">libuuid-devel</packagereq>
      <packagereq type="mandatory">lksctp-tools-devel</packagereq>
      <packagereq type="mandatory">lz4</packagereq>
      <packagereq type="mandatory">lz4-devel</packagereq>
      <packagereq type="mandatory">lzo</packagereq>
      <packagereq type="mandatory">lzo-devel</packagereq>
      <packagereq type="mandatory">numactl-devel</packagereq>
      <packagereq type="mandatory">pciutils-devel</packagereq>
      <packagereq type="mandatory">pcre-devel</packagereq>
      <packagereq type="mandatory">polkit-devel</packagereq>
      <packagereq type="mandatory">popt-devel</packagereq>
      <packagereq type="mandatory">rdma-core-devel</packagereq>
      <packagereq type="mandatory">readline-devel</packagereq>
      <packagereq type="mandatory">sqlite-devel</packagereq>
      <packagereq type="mandatory">systemd-devel</packagereq>
      <packagereq type="mandatory">tcl-devel</packagereq>
      <packagereq type="mandatory">xfsprogs-devel</packagereq>
      <packagereq type="mandatory">xz-devel</packagereq>
    </packagelist>
  </group>
  <group>
    <id>platform-devel</id>
    <name>Platform Development</name>
    <name xml:lang="as">প্লেটফৰ্ম উন্নয়ন</name>
    <name xml:lang="cs">Vývoj platformy</name>
    <name xml:lang="de">Plattformentwicklung</name>
    <name xml:lang="de_CH">Plattformentwicklung</name>
    <name xml:lang="es">Desarrollo de plataforma</name>
    <name xml:lang="fr">Développement plateforme</name>
    <name xml:lang="gu">પ્લેટફોર્મ વિકાસ</name>
    <name xml:lang="hi">प्लैटफॉर्म विकास</name>
    <name xml:lang="it">Sviluppo piattaforma</name>
    <name xml:lang="ja">プラットフォーム開発</name>
    <name xml:lang="kn">ಪ್ಲಾಟ್‌ಫಾರ್ಮ್ ವಿಕಸನ</name>
    <name xml:lang="ko">플랫폼 개발</name>
    <name xml:lang="ml">പ്ലാറ്റ്ഫോം വികസനം</name>
    <name xml:lang="mr">प्लॅटफॉर्म डेव्हलपमेंट</name>
    <name xml:lang="or">ପ୍ଲାଟଫର୍ମ ବିକାଶ</name>
    <name xml:lang="pa">ਮੰਚ ਵਿਕਾਸ</name>
    <name xml:lang="pl">Rozwijanie platformy</name>
    <name xml:lang="pt">Desenvolvimento de Plataforma</name>
    <name xml:lang="pt_BR">Desenvolvimento de Plataforma</name>
    <name xml:lang="ru">Средства разработки платформы</name>
    <name xml:lang="ta">இயங்கு தள உருவாக்கம்</name>
    <name xml:lang="te">ప్లాట్‌ఫాం అభివృద్ది</name>
    <name xml:lang="uk">Розробка для платформи</name>
    <name xml:lang="zh">平台開發</name>
    <name xml:lang="zh_CN">平台开发</name>
    <name xml:lang="zh_TW">平台開發</name>
    <description>Recommended development headers and libraries for developing applicat</description>
    <default>true</default>
    <uservisible>false</uservisible>
    <packagelist>
      <packagereq type="mandatory">fontconfig-devel</packagereq>
      <packagereq type="mandatory">freetype-devel</packagereq>
      <packagereq type="mandatory">glib2-devel</packagereq>
      <packagereq type="mandatory">glibc-devel</packagereq>
      <packagereq type="mandatory">krb5-devel</packagereq>
      <packagereq type="mandatory">libpng-devel</packagereq>
      <packagereq type="mandatory">ncurses-devel</packagereq>
      <packagereq type="mandatory">openldap-devel</packagereq>
      <packagereq type="mandatory">openssl-devel</packagereq>
      <packagereq type="mandatory">pam-devel</packagereq>
      <packagereq type="mandatory">zlib-devel</packagereq>
    </packagelist>
  </group>
  <group>
    <id>python-web</id>
    <name>Python Web</name>
    <name xml:lang="de">Python Web</name>
    <name xml:lang="es">Python Web</name>
    <name xml:lang="fr">Python Web</name>
    <name xml:lang="it">Python Web</name>
    <name xml:lang="ja">Python Web</name>
    <name xml:lang="ko">Python Web</name>
    <name xml:lang="pt_BR">Python Web</name>
    <name xml:lang="ru">Python Web</name>
    <name xml:lang="zh_CN">Python Web</name>
    <name xml:lang="zh_TW">Python Web</name>
    <description>Basic Python web application support.</description>
    <description xml:lang="as">মৌলিক Python ৱেব এপ্লিকেচন সমৰ্থন।</description>
    <description xml:lang="cs">Podpora pro základní webové aplikace v jazyce Python.</description>
    <description xml:lang="de">Grundlegende Unterstützung für die Python Webanwendung.</description>
    <description xml:lang="de_CH">Grundlegende Unterstützung für die Python Webanwendu</description>
    <description xml:lang="es">Soporte de aplicaciones web en Basic Python</description>
    <description xml:lang="fr">Prise en charge d'application web Python de base.</description>
    <description xml:lang="gu">મૂળભૂત Python વેબ કાર્યક્રમ આધાર.</description>
    <description xml:lang="hi">बेसिक पाइथन वेब अनुप्रयोग समर्थन.</description>
    <description xml:lang="it">Supporto applicazione web Python di base</description>
    <description xml:lang="ja">Python のベーシック web アプリケーションサポートです。</description>
    <description xml:lang="kn">ಮೂಲಭೂತ Python ಜಾಲ ಅನ್ವಯ ಬೆಂಬಲ.</description>
    <description xml:lang="ko">기본 Python 웹 애플리케이션 지원.</description>
    <description xml:lang="ml">അടിസ്ഥാന പൈഥണ്‍ വെബ് പ്രയോഗത്തിനുള്ള പിന്തുണ.</description>
    <description xml:lang="mr">बेसिक Python वेब ॲप्लिकेशन समर्थन.</description>
    <description xml:lang="or">ମୌଳିକ Python ୱେବ ପ୍ରୟୋଗ ସହାୟତା।</description>
    <description xml:lang="pa">ਮੁੱਢਲਾ ਪਾਇਥਨ ਵੈੱਬ ਐਪਲੀਕੇਸ਼ਨ ਸਮਰਥਨ।</description>
    <description xml:lang="pl">Podstawowa obsługa aplikacji WWW w języku Python.</description>
    <description xml:lang="pt">Suporte de aplicativo da Web Phyton Básico</description>
    <description xml:lang="pt_BR">Suporte de aplicativo da Web Phyton Básico</description>
    <description xml:lang="ru">Базовая поддержка веб-приложений Python.</description>
    <description xml:lang="ta">அடிப்படை Python வலைப் பயன்பாடு ஆதரவு.</description>
    <description xml:lang="te">సాధారణ Python వెబ్ అనువర్తన తోడ్పాటు.</description>
    <description xml:lang="uk">Базова підтримка вебпрограм мовою Python.</description>
    <description xml:lang="zh">基本的 Python 網站應用程式支援。</description>
    <description xml:lang="zh_CN">基本 Python 的 web 应用程序支持。</description>
    <description xml:lang="zh_TW">基本的 Python 網站應用程式支援。</description>
    <default>true</default>
    <uservisible>false</uservisible>
    <packagelist>
      <packagereq type="mandatory">freetype-devel</packagereq>
      <packagereq type="mandatory">libcurl</packagereq>
      <packagereq type="mandatory">libcurl-devel</packagereq>
      <packagereq type="mandatory">python3-magic</packagereq>
    </packagelist>
  </group>
  <category>
    <id>development</id>
    <name>Development</name>
    <name xml:lang="af">Ontwikkeling</name>
    <name xml:lang="am">እድገት</name>
    <name xml:lang="ar">تطوير</name>
    <name xml:lang="as">উন্নয়ন</name>
    <name xml:lang="bal">پیش روگ</name>
    <name xml:lang="be">Распрацоўка</name>
    <name xml:lang="bg">Разработка</name>
    <name xml:lang="bn">সফ্টওয়্যার নির্মাণ</name>
    <name xml:lang="bs">Razvoj</name>
    <name xml:lang="ca">Desenvolupament</name>
    <name xml:lang="cs">Vývoj</name>
    <name xml:lang="cy">Datblygiad</name>
    <name xml:lang="da">Udvikling</name>
    <name xml:lang="de">Softwareentwicklung</name>
    <name xml:lang="de_CH">Softwareentwicklung</name>
    <name xml:lang="el">Ανάπτυξη λογισμικού</name>
    <name xml:lang="en_GB">Development</name>
    <name xml:lang="es">Desarrollo</name>
    <name xml:lang="et">Arendus</name>
    <name xml:lang="eu">Garapena</name>
    <name xml:lang="fa">برنامه‌سازی</name>
    <name xml:lang="fa_IR">برنامه‌سازی</name>
    <name xml:lang="fi">Ohjelmistokehitys</name>
    <name xml:lang="fr">Développement</name>
    <name xml:lang="gl">Desenvolvemento</name>
    <name xml:lang="gu">વિકાસ</name>
    <name xml:lang="he">פיתוח</name>
    <name xml:lang="hi">विकास</name>
    <name xml:lang="hr">Razvoj</name>
    <name xml:lang="hu">Fejlesztőeszközök</name>
    <name xml:lang="hy">Ծրագրավորում</name>
    <name xml:lang="ia">Disveloppamento</name>
    <name xml:lang="id">Pengembangan</name>
    <name xml:lang="ilo">Panagbukel</name>
    <name xml:lang="is">Þróun</name>
    <name xml:lang="it">Sviluppo</name>
    <name xml:lang="ja">開発</name>
    <name xml:lang="ka">პრიოგრამირება</name>
    <name xml:lang="kn">ವಿಕಾಸನ</name>
    <name xml:lang="ko">개발</name>
    <name xml:lang="lv">Programmēšana</name>
    <name xml:lang="mai">विकास</name>
    <name xml:lang="mk">Развој</name>
    <name xml:lang="ml">ഡെവലപ്പ്മെന്‍റ്</name>
    <name xml:lang="mr">विकास</name>
    <name xml:lang="ms">Pembangunan</name>
    <name xml:lang="nb">Utviklingsverktøy</name>
    <name xml:lang="ne">विकास</name>
    <name xml:lang="nl">Ontwikkeling</name>
    <name xml:lang="no">Utviklingsverktøy</name>
    <name xml:lang="nso">Tšwetšopele</name>
    <name xml:lang="or">ବିକାଶ</name>
    <name xml:lang="pa">ਵਿਕਾਸ</name>
    <name xml:lang="pl">Programowanie</name>
    <name xml:lang="pt">Desenvolvimento</name>
    <name xml:lang="pt_BR">Desenvolvimento</name>
    <name xml:lang="ro">Dezvoltare</name>
    <name xml:lang="ru">Программирование</name>
    <name xml:lang="si">ප්‍රවර්ධනය</name>
    <name xml:lang="sk">Vývoj</name>
    <name xml:lang="sl">Razvoj</name>
    <name xml:lang="sq">Zhvillim</name>
    <name xml:lang="sr">Развој</name>
    <name xml:lang="sr@Latn">Razvoj</name>
    <name xml:lang="sr@latin">Razvoj</name>
    <name xml:lang="sv">Utveckling</name>
    <name xml:lang="ta">உருவாக்கம்</name>
    <name xml:lang="te">అభివృద్ధి</name>
    <name xml:lang="tg">Барноманависӣ</name>
    <name xml:lang="th">การพัฒนาซอฟต์แวร์</name>
    <name xml:lang="tr">Geliştirme</name>
    <name xml:lang="uk">Програмування</name>
    <name xml:lang="ur">تمیرات</name>
    <name xml:lang="vi">Phát triển</name>
    <name xml:lang="zh">程式開發</name>
    <name xml:lang="zh_CN">开发</name>
    <name xml:lang="zh_TW">程式開發</name>
    <name xml:lang="zu">Intuthuko</name>
    <description>Packages which provide functionality for developing and building appl</description>
    <description xml:lang="ar">تقدم الحزم امكانيات تطوير و بناء تطبيقات.</description>
    <description xml:lang="bal">بستگانی که په پیشروگ و شرکنگ کارمرزی عمل آرنت.</description>
    <description xml:lang="bg">Пакети предоставящи функционалност необходима за разраб</description>
    <description xml:lang="bn">অ্যাপ্লিকেশন ডিভেলপমেন্ট ও বিল্ড করতে সহায়ক প্যাকেজ।</description>
    <description xml:lang="bs">Paketi koji pružaju funkcionalnosti za razvoj i izgradn</description>
    <description xml:lang="ca">Paquets que proporcionen funcionalitats per a desenvolu</description>
    <description xml:lang="cs">Balíčky poskytující podporu pro vývoj a kompilaci aplik</description>
    <description xml:lang="da">Pakker som giver funktionalitet for udvikling og bygnin</description>
    <description xml:lang="de">Pakete, die Funktionalität für die Entwicklung und die </description>
    <description xml:lang="de_CH">Pakete, die Funktionalität für die Entwicklung und d</description>
    <description xml:lang="el">Πακέτα τα οποία προσφέρουν λειτουργικότητα για ανάπτυξη</description>
    <description xml:lang="en_GB">Packages which provide functionality for developing </description>
    <description xml:lang="es">Paquetes que proveen la funcionalidad para desarrollar </description>
    <description xml:lang="et">Rakenduste arendamiseks ja kompileerimiseks vajalik fun</description>
    <description xml:lang="fi">Paketit, jotka tarjoavat tuen sovellusten kehittämisell</description>
    <description xml:lang="fr">Packages qui offrent des fonctions de développement et </description>
    <description xml:lang="he">חבילות המאפשרות פיתוח ובניית תוכנה.</description>
    <description xml:lang="hr">Paketi koji pružaju funkcionalnosti za razvoj i izgradn</description>
    <description xml:lang="hu">Alkalmazások fejlesztésére szolgáló csomagok</description>
    <description xml:lang="ia">Pacchettos que provide functionalitate pro disveloppar</description>
    <description xml:lang="id">Paket yang menyediakan fungsionalitas untuk mengembangk</description>
    <description xml:lang="is">Pakkar sem innihalda virkni til að þróa og smíða hugbún</description>
    <description xml:lang="it">Pacchetti che forniscono funzionalità per sviluppare e </description>
    <description xml:lang="ja">アプリケーションの開発および構築機能を提供するパッケージです。</description>
    <description xml:lang="ko">이 패키지는 응용 프로그램을 개발하기 위한 기능을 제공합니다.</description>
    <description xml:lang="lv">Pakotnes, kas nodrošina programmu izstrādes procesu.</description>
    <description xml:lang="ms">Pakej yang menyediakan fungsi untuk membangun dan memb</description>
    <description xml:lang="nb">Pakker som gir funksjonalitet for utvikling og bygging </description>
    <description xml:lang="nl">Pakketten die functionaliteit bieden voor het ontwikkel</description>
    <description xml:lang="pl">Pakiety dostarczające funkcjonalność tworzenia i budowa</description>
    <description xml:lang="pt">Pacotes que oferecem funcionalidades para desenvolver e</description>
    <description xml:lang="pt_BR">Pacotes que oferecem funcionalidades para desenvolve</description>
    <description xml:lang="ro">Pachete care oferă funcţionalităţi pentru dezvoltarea ş</description>
    <description xml:lang="ru">Пакеты, предоставляющие возможность разработки и сборки</description>
    <description xml:lang="sk">Balíčky poskytujúce funkcionalitu pre vývoj a zostavova</description>
    <description xml:lang="sl">Paketi za razvoj programja.</description>
    <description xml:lang="sq">Paketa që ofrojnë funksione për zhvillim dhe ndërtim zb</description>
    <description xml:lang="sr">Пакети који пружају функционалност за развој и изградњу</description>
    <description xml:lang="sr@Latn">Paketi koji pružaju funkcionalnost za razvoj i i</description>
    <description xml:lang="sr@latin">Paketi koji pružaju funkcionalnost za razvoj i i</description>
    <description xml:lang="sv">Paket som tillhandahåller funktionalitet för utveckling</description>
    <description xml:lang="tg">Барномаҳои барноманависӣ ва эҷодкунии худи барномаҳо</description>
    <description xml:lang="th">ชุดแพกเกจที่ใช้สำหรับการพัฒนาและสร้างโปรแกรมประยุกต์</description>
    <description xml:lang="tr">Uygulama geliştirmek ve oluşturmak için gerekli işlevle</description>
    <description xml:lang="uk">Пакунки, що надають можливість розробляти та збирати пр</description>
    <description xml:lang="zh">提供開發及建立應用程式功能的套件</description>
    <description xml:lang="zh_CN">用于开发和构建应用程序的软件包。</description>
    <description xml:lang="zh_TW">提供開發及建立應用程式功能的套件</description>
    <display_order>90</display_order>
    <grouplist>
      <groupid>additional-devel</groupid>
      <groupid>development</groupid>
      <groupid>platform-devel</groupid>
    </grouplist>
  </category>
  <category>
    <id>servers</id>
    <name>Servers</name>
    <name xml:lang="af">Bedieners</name>
    <name xml:lang="am">ሰርቨሮች</name>
    <name xml:lang="ar">خوادم</name>
    <name xml:lang="as">চাৰ্ভাৰসমূহ</name>
    <name xml:lang="bal">سروران</name>
    <name xml:lang="be">Паслужнікі</name>
    <name xml:lang="bg">Сървъри</name>
    <name xml:lang="bn">সার্ভার</name>
    <name xml:lang="bs">Poslužitelji</name>
    <name xml:lang="ca">Servidors</name>
    <name xml:lang="cs">Servery</name>
    <name xml:lang="cy">Gweinyddion</name>
    <name xml:lang="da">Servere</name>
    <name xml:lang="de">Server</name>
    <name xml:lang="de_CH">Server</name>
    <name xml:lang="el">Εξυπηρετητές</name>
    <name xml:lang="en_GB">Servers</name>
    <name xml:lang="es">Servidores</name>
    <name xml:lang="et">Serverid</name>
    <name xml:lang="eu">Zerbitzariak</name>
    <name xml:lang="fa">کارگزارها</name>
    <name xml:lang="fa_IR">کارگزارها</name>
    <name xml:lang="fi">Palvelimet</name>
    <name xml:lang="fr">Serveurs</name>
    <name xml:lang="gl">Servidores</name>
    <name xml:lang="gu">સર્વરો</name>
    <name xml:lang="he">שרתים</name>
    <name xml:lang="hi">सर्वर</name>
    <name xml:lang="hr">Poslužitelji</name>
    <name xml:lang="hu">Kiszolgálók</name>
    <name xml:lang="hy">Սերվերներ</name>
    <name xml:lang="ia">Servitores</name>
    <name xml:lang="id">Server</name>
    <name xml:lang="ilo">Serbidor</name>
    <name xml:lang="is">Þjónar</name>
    <name xml:lang="it">Servers</name>
    <name xml:lang="ja">サーバー</name>
    <name xml:lang="ka">სერვერები</name>
    <name xml:lang="kn">ಪೂರೈಕೆಗಣಕಗಳು</name>
    <name xml:lang="ko">서버</name>
    <name xml:lang="lv">Serveri</name>
    <name xml:lang="mai">सर्वर</name>
    <name xml:lang="mk">Сервери</name>
    <name xml:lang="ml">സര്‍വറുകള്‍</name>
    <name xml:lang="mr">सेवक</name>
    <name xml:lang="ms">Pelayan-Pelayan</name>
    <name xml:lang="nb">Tjenere</name>
    <name xml:lang="ne">सर्भर</name>
    <name xml:lang="nl">Servers</name>
    <name xml:lang="no">Tjenere</name>
    <name xml:lang="nso">Baabi</name>
    <name xml:lang="or">ସେବକ ମାନେ</name>
    <name xml:lang="pa">ਸਰਵਰ</name>
    <name xml:lang="pl">Serwery</name>
    <name xml:lang="pt">Servidores</name>
    <name xml:lang="pt_BR">Servidores</name>
    <name xml:lang="ro">Servere</name>
    <name xml:lang="ru">Серверы</name>
    <name xml:lang="si">සේවා දායකයන්</name>
    <name xml:lang="sk">Servery</name>
    <name xml:lang="sl">Strežniki</name>
    <name xml:lang="sq">Shërbyesa</name>
    <name xml:lang="sr">Сервери</name>
    <name xml:lang="sr@Latn">Serveri</name>
    <name xml:lang="sr@latin">Serveri</name>
    <name xml:lang="sv">Servrar</name>
    <name xml:lang="ta">சேவையகங்கள்</name>
    <name xml:lang="te">సేవికలు</name>
    <name xml:lang="tg">Хидматгоҳҳо</name>
    <name xml:lang="th">ตัวให้บริการ</name>
    <name xml:lang="tr">Sunucular</name>
    <name xml:lang="uk">Сервери</name>
    <name xml:lang="ur">سرورز</name>
    <name xml:lang="vi">Server</name>
    <name xml:lang="zh">伺服器</name>
    <name xml:lang="zh_CN">服务器</name>
    <name xml:lang="zh_TW">伺服器</name>
    <name xml:lang="zu">Abalekeleli</name>
    <description>Software used for running network servers</description>
    <description xml:lang="ar">يستخدم البرنامج لادارة خادمات الشبكة</description>
    <description xml:lang="as">নেটৱাৰ্ক চাৰ্ভাৰ চলাব'লৈ চফ্টৱেৰ</description>
    <description xml:lang="bal">برنامه ای که په اجرای سروران شبکه ای استفاده بیت.</description>
    <description xml:lang="bg">Софтуер за пускане на мрежови услуги</description>
    <description xml:lang="bn">নেটওয়ার্ক সার্ভার সঞ্চালনের উদ্দেশ্যে সফ্টওয়্যার</description>
    <description xml:lang="bs">Softver koji se upotrebljava za pokretanje mrežnih posl</description>
    <description xml:lang="ca">Programari emprat per executar servidors de xarxa</description>
    <description xml:lang="cs">Software používaný pro běh síťových serverů</description>
    <description xml:lang="da">Programmel til brug for at køre netværksservere</description>
    <description xml:lang="de">Software, die für den Betrieb von Netzwerkservern verwe</description>
    <description xml:lang="de_CH">Software, die für den Betrieb von Netzwerkservern ve</description>
    <description xml:lang="el">Λογισμικό που χρησιμοποιείται για εκτέλεση δικτυακών εξ</description>
    <description xml:lang="en_GB">Software used for running network servers</description>
    <description xml:lang="es">Programas usados para correr servidores de red</description>
    <description xml:lang="et">Võrguserverite töölepanemiseks vajalik tarkvara</description>
    <description xml:lang="fi">Verkkopalvelimina käytettäviä ohjelmistoja</description>
    <description xml:lang="fr">Logiciels utilisés pour serveurs de réseau actifs</description>
    <description xml:lang="gu">નેટવર્ક સર્વરો ચલાવવા માટે વપરાતું સોફ્ટવેર</description>
    <description xml:lang="he">תוכנה המשמשת להרצת שרתי רשת</description>
    <description xml:lang="hi">संजाल सर्वर के लिये प्रयोग के लिये सॉफ्टवेयर</description>
    <description xml:lang="hr">Softver koji se upotrebljava za pokretanje mrežnih posl</description>
    <description xml:lang="hu">Hálózati kiszolgálók futtatásához szükséges szoftverek</description>
    <description xml:lang="hy">Ծրագրեր՝ ցանցային սերվերներ աշխատեցնելու համար</description>
    <description xml:lang="ia">Software usate pro exequer servitores de rete</description>
    <description xml:lang="id">Perangkat lunak yang dipergunakan untuk menjalankan ser</description>
    <description xml:lang="is">Hugbúnaður til að keyra netþjóna</description>
    <description xml:lang="it">Software usato per eseguire server di rete</description>
    <description xml:lang="ja">ネットワークサーバーの稼動に使用するソフトウェア</description>
    <description xml:lang="kn">ಜಾಲ ಪೂರೈಕೆಗಣಕಗಳನ್ನು ಕಾರ್ಯಗತಗೊಳಿಸಲು ಬಳಸುವ ತಂತ್ರಾಂಶ</description>
    <description xml:lang="ko">네트워크 서버를 실행하기 위해 사용되는 소프트웨어</description>
    <description xml:lang="lv">Programmas, kas tiek izmantotas tīkla serveriem</description>
    <description xml:lang="mai">संजाल सर्वर क' लेल प्रयोग क' लेल साफ्टवेयर</description>
    <description xml:lang="mr">संजाळ सर्व्हर चालवण्यासाठी वापरले जाणारे सॉफ्टवेअर</description>
    <description xml:lang="ms">Perisian digunakan untuk melaksanakan pelayan rangkaian</description>
    <description xml:lang="nb">Programvare som brukes for å kjøre nettverkstjenere</description>
    <description xml:lang="ne">सञ्जाल सर्भर चलाउनका लागि प्रयोग गरिने सफ्टवेयर</description>
    <description xml:lang="nl">Software gebruikt voor het uitvoeren van netwerk-server</description>
    <description xml:lang="or">ନେଟୱାର୍କ ସେବକକୁ ଚଳାଇବା ପାଇଁ ବ୍ଯବହୃତ ସଫଟୱେର</description>
    <description xml:lang="pa">ਨੈੱਟਵਰਕ ਸਰਵਰ ਚਲਾਉਣ ਲਈ ਲੋੜੀਦੇ ਸਾਫਟਵੇਅਰ</description>
    <description xml:lang="pl">Oprogramowanie używane do uruchamiania serwerów sieciow</description>
    <description xml:lang="pt">Aplicações usadas para executar servidores de rede</description>
    <description xml:lang="pt_BR">Aplicações usadas para executar servidores de rede</description>
    <description xml:lang="ro">Programe folosite pentru a rula servere de reţea</description>
    <description xml:lang="ru">Программы, используемые для запуска сетевых серверов</description>
    <description xml:lang="sk">Softvér používaný na beh sieťových serverov</description>
    <description xml:lang="sl">Programje za delo z omrežnimi strežniki</description>
    <description xml:lang="sq">Programe të përdorur për xhirim shërbyesash rrjeti</description>
    <description xml:lang="sr">Софтвер који се користи за извршавање мрежних сервера</description>
    <description xml:lang="sr@Latn">Softver koji se koristi za izvršavanje mrežnih ser</description>
    <description xml:lang="sr@latin">Softver koji se koristi za izvršavanje mrežnih se</description>
    <description xml:lang="sv">Mjukvara som används för körande nätverksservrar</description>
    <description xml:lang="ta">பிணைய சேவையகங்களை இயக்க பயன்படுத்தப்பட்ட மென்பொருள்</description>
    <description xml:lang="te">నెట్వర్క్‍ సేవికలను నడుపుటకు ఉపయోగపడే సాఫ్ట్‍‌వేర్</description>
    <description xml:lang="tg">Нармафзори истифодашудаи бо хидматгоҳҳои Интернет</description>
    <description xml:lang="th">โปรแกรมสำหรับให้บริการในเครือข่าย</description>
    <description xml:lang="tr">Ağ servislerini çalıştırmak için kullanılan yazılımlar</description>
    <description xml:lang="uk">Програми, що використовуються для запуску мережних серв</description>
    <description xml:lang="zh">用於運行網路伺服器的軟體</description>
    <description xml:lang="zh_CN">用于运行网络服务器的软件</description>
    <description xml:lang="zh_TW">用於運行網路伺服器的軟體</description>
    <display_order>20</display_order>
    <grouplist>
      <groupid>file-server</groupid>
      <groupid>mail-server</groupid>
      <groupid>network-server</groupid>
    </grouplist>
  </category>
  <category>
    <id>base-system</id>
    <name>System</name>
    <name xml:lang="af">Rekenaar</name>
    <name xml:lang="am">ሲስተም</name>
    <name xml:lang="ar">نظام</name>
    <name xml:lang="as">চিস্টেম</name>
    <name xml:lang="be">Сытэма</name>
    <name xml:lang="bn">সিস্টেম</name>
    <name xml:lang="bs">Sustav</name>
    <name xml:lang="ca">Sistema</name>
    <name xml:lang="cs">Systém</name>
    <name xml:lang="cy">System</name>
    <name xml:lang="de">System</name>
    <name xml:lang="de_CH">System</name>
    <name xml:lang="en_GB">System</name>
    <name xml:lang="es">Sistema</name>
    <name xml:lang="eu">Sistema</name>
    <name xml:lang="fa">سیستم</name>
    <name xml:lang="fa_IR">سیستم</name>
    <name xml:lang="fi">Järjestelmä</name>
    <name xml:lang="fr">Système</name>
    <name xml:lang="gl">Sistema</name>
    <name xml:lang="gu">સિસ્ટમ</name>
    <name xml:lang="hi">तंत्र</name>
    <name xml:lang="hr">Sustav</name>
    <name xml:lang="hu">Rendszer</name>
    <name xml:lang="hy">Համակարգ</name>
    <name xml:lang="id">Sistem</name>
    <name xml:lang="ilo">Sistema</name>
    <name xml:lang="it">Sistema</name>
    <name xml:lang="ja">システム</name>
    <name xml:lang="ka">სისტემა</name>
    <name xml:lang="kn">ವ್ಯವಸ್ಥೆ</name>
    <name xml:lang="ko">시스템</name>
    <name xml:lang="lv">Sistēma</name>
    <name xml:lang="mk">Систем</name>
    <name xml:lang="ml">സിസ്റ്റം</name>
    <name xml:lang="mr">प्रणाली</name>
    <name xml:lang="no">System</name>
    <name xml:lang="nso">Tshepedišo</name>
    <name xml:lang="or">ତନ୍ତ୍ର</name>
    <name xml:lang="pa">ਸਿਸਟਮ</name>
    <name xml:lang="pl">System</name>
    <name xml:lang="pt">Sistema </name>
    <name xml:lang="pt_BR">Sistema </name>
    <name xml:lang="ru">Система</name>
    <name xml:lang="si">පද්ධතිය</name>
    <name xml:lang="sl">Sistem</name>
    <name xml:lang="sq">Sistem</name>
    <name xml:lang="ta">கணினி</name>
    <name xml:lang="te">వ్యవస్థ</name>
    <name xml:lang="th">ระบบ</name>
    <name xml:lang="tr">Sistem</name>
    <name xml:lang="uk">Система</name>
    <name xml:lang="ur">نظام</name>
    <name xml:lang="vi">Hệ thống</name>
    <name xml:lang="zh">系統</name>
    <name xml:lang="zh_CN">系统</name>
    <name xml:lang="zh_TW">系統</name>
    <name xml:lang="zu">Isistimu</name>
    <description>Core system components.</description>
    <description xml:lang="as">চিস্টেমৰ মূখ্য উপাদানসমূহ।</description>
    <description xml:lang="bn">সিস্টেমের কোর সামগ্রী।</description>
    <description xml:lang="cs">Základní komponenty systému.</description>
    <description xml:lang="de">Zentrale Systemkomponenten.</description>
    <description xml:lang="de_CH">Zentrale Systemkomponenten.</description>
    <description xml:lang="es">Componentes de sistema Core</description>
    <description xml:lang="fr">Composants du système de base.</description>
    <description xml:lang="gu">કોર સિસ્ટમ સાધનો.</description>
    <description xml:lang="hi">प्रधान सिस्टम घटक</description>
    <description xml:lang="it">Componenti di base del sistema.</description>
    <description xml:lang="ja">コアシステムコンポーネント。</description>
    <description xml:lang="kn">ಪ್ರಮುಖ ವ್ಯವಸ್ಥೆಯ ಘಟಕಗಳು.</description>
    <description xml:lang="ko">핵심 시스템 콤포넌트.</description>
    <description xml:lang="ml">കോര്‍ സിസ്റ്റം ഘടകങ്ങള്‍.</description>
    <description xml:lang="mr">कोर प्रणाली घटके.</description>
    <description xml:lang="or">ମୂଖ୍ୟ ତନ୍ତ୍ର ଉପାଦାନଗୁଡ଼ିକ।</description>
    <description xml:lang="pa">ਮੁੱਖ ਸਿਸਟਮ ਹਿੱਸੇ।</description>
    <description xml:lang="pl">Główne składniki systemu.</description>
    <description xml:lang="pt">Componentes de sistema central</description>
    <description xml:lang="pt_BR">Componentes de sistema central</description>
    <description xml:lang="ru">Основные компоненты системы.</description>
    <description xml:lang="sv">Grundläggande systemkomponenter.</description>
    <description xml:lang="ta">உள்ளீடு கணினி ஆக்கக்கூறுகள்.</description>
    <description xml:lang="te">కోర్ సిస్టమ్ మూలకములు.</description>
    <description xml:lang="uk">Основні компонент систем.</description>
    <description xml:lang="zh">核心系統元件。</description>
    <description xml:lang="zh_CN">核新系统组件。</description>
    <description xml:lang="zh_TW">核心系統元件。</description>
    <display_order>10</display_order>
    <grouplist>
      <groupid>dial-up</groupid>
      <groupid>hardware-monitoring</groupid>
      <groupid>infiniband</groupid>
      <groupid>large-systems</groupid>
      <groupid>legacy-unix</groupid>
      <groupid>mainframe-access</groupid>
      <groupid>network-tools</groupid>
      <groupid>performance</groupid>
      <groupid>scientific</groupid>
      <groupid>security-tools</groupid>
      <groupid>smart-card</groupid>
      <groupid>standard</groupid>
    </grouplist>
  </category>
  <category>
    <id>desktops</id>
    <name>Desktops</name>
    <name xml:lang="af">Werkskerms</name>
    <name xml:lang="am">ዴስክቶፖች</name>
    <name xml:lang="ar">أسطح المكتب</name>
    <name xml:lang="as">ডেস্কটপ</name>
    <name xml:lang="be">Стальцы</name>
    <name xml:lang="bn">ডেস্কটপ</name>
    <name xml:lang="bs">Radne površine</name>
    <name xml:lang="ca">Escriptoris</name>
    <name xml:lang="cs">Pracovní prostředí</name>
    <name xml:lang="cy">Penbyrddau</name>
    <name xml:lang="de">Desktops</name>
    <name xml:lang="de_CH">Desktops</name>
    <name xml:lang="en_GB">Desktops</name>
    <name xml:lang="es">Escritorios</name>
    <name xml:lang="eu">Mahaigainak</name>
    <name xml:lang="fa">محیط‌های رومیزی</name>
    <name xml:lang="fa_IR">محیط‌های رومیزی</name>
    <name xml:lang="fi">Työpöydät</name>
    <name xml:lang="fr">Bureaux</name>
    <name xml:lang="gl">Escritorios</name>
    <name xml:lang="gu">ડેસ્કટોપો</name>
    <name xml:lang="hi">डेस्कटॉप</name>
    <name xml:lang="hr">Radne površine</name>
    <name xml:lang="hu">Munkaasztalok</name>
    <name xml:lang="hy">Դեսքթոփներ</name>
    <name xml:lang="ia">Scriptorios</name>
    <name xml:lang="id">Desktop</name>
    <name xml:lang="ilo">Eskritorio</name>
    <name xml:lang="it">Desktop</name>
    <name xml:lang="ja">デスクトップ</name>
    <name xml:lang="ka">სამუშაო გარემოები</name>
    <name xml:lang="kn">ಗಣಕತೆರೆಗಳು</name>
    <name xml:lang="ko">데스크탑</name>
    <name xml:lang="lv">Darbvirsmas</name>
    <name xml:lang="mk">Графички работни околини</name>
    <name xml:lang="ml">പണിയിടങ്ങള്‍</name>
    <name xml:lang="mr">डेस्कटॉप्स्</name>
    <name xml:lang="no">Skrivebord</name>
    <name xml:lang="nso">Diteseke</name>
    <name xml:lang="or">ଡେସ୍କଟପ୍ ମାନ</name>
    <name xml:lang="pa">ਡੈਸਕਟਾਪ</name>
    <name xml:lang="pl">Środowiska graficzne</name>
    <name xml:lang="pt">Desktops</name>
    <name xml:lang="pt_BR">Desktops</name>
    <name xml:lang="ru">Рабочие столы</name>
    <name xml:lang="si">Desktops</name>
    <name xml:lang="sl">Namizja</name>
    <name xml:lang="sq">Desktopë</name>
    <name xml:lang="sv">Skrivbord</name>
    <name xml:lang="ta">பணிமேடைகள்</name>
    <name xml:lang="te">డెస్కుటాప్స్</name>
    <name xml:lang="th">เดสก์ท็อป</name>
    <name xml:lang="tr">Masaüstleri</name>
    <name xml:lang="uk">Робочі середовища</name>
    <name xml:lang="ur">ڈيسک ٹاپس</name>
    <name xml:lang="vi">Desktop</name>
    <name xml:lang="zh">桌面環境</name>
    <name xml:lang="zh_CN">桌面</name>
    <name xml:lang="zh_TW">桌面環境</name>
    <name xml:lang="zu">Amadesktop</name>
    <description>Desktops and thin clients.</description>
    <description xml:lang="as">ডেস্কটপ আৰু ক্ষীণ ক্লাএন্ট।</description>
    <description xml:lang="bn">ডেস্কটপ ও থিন ক্লায়েন্ট।</description>
    <description xml:lang="cs">Pracovní prostředí a tencí klienti.</description>
    <description xml:lang="de">Desktops und Thin-Clients.</description>
    <description xml:lang="de_CH">Desktops und Thin-Clients.</description>
    <description xml:lang="es">Escritorios y clientes livianos.</description>
    <description xml:lang="fr">Bureaux et clients légers.</description>
    <description xml:lang="gu">ડેસ્કટોપ અને પાતળા ક્લાયન્ટો.</description>
    <description xml:lang="hi">डेस्कटॉप और थिन क्लाइंट</description>
    <description xml:lang="ia">Scriptorios e clientes legier.</description>
    <description xml:lang="it">Desktop e thin client.</description>
    <description xml:lang="ja">デスクトップとシンクライアント。</description>
    <description xml:lang="kn">ಗಣಕತೆರೆಗಳು ಹಾಗು ಥಿನ್‌ ಕ್ಲೈಂಟ್‌ಗಳು.</description>
    <description xml:lang="ko">데스크탑과 씬 클라이언트.</description>
    <description xml:lang="ml">ഡസ്ക്ടോപ്പുകളും ഥിന്‍ ക്ലയന്റുകളും്റുകള്‍</description>
    <description xml:lang="mr">डेस्कटॉप्स् व थीन क्लाएंट्स्.</description>
    <description xml:lang="or">ଡେସ୍କଟପ ଏବଂ ଥିନ କ୍ଲାଏଣ୍ଟମାନ।</description>
    <description xml:lang="pa">ਡੈਸਕਟਾਪ ਅਤੇ ਥਿੰਨ ਕਲਾਈਂਟ</description>
    <description xml:lang="pl">Środowiska graficzne i ciency klienci.</description>
    <description xml:lang="pt">Ambientes de Trabalho e clientes finos</description>
    <description xml:lang="pt_BR">Ambientes de Trabalho e clientes finos</description>
    <description xml:lang="ru">Рабочие столы и ?тонкие? клиенты.</description>
    <description xml:lang="sv">Skrivbord och tunna klienter.</description>
    <description xml:lang="ta">பணிமேடை மற்றும் ஒல்லியான வாடிக்கையாளர்கள்.</description>
    <description xml:lang="te">డెస్కుటాప్స్ మరియు తిన్ క్లైంట్లు.</description>
    <description xml:lang="uk">Робочі станції та «тонкі» клієнти.</description>
    <description xml:lang="zh">桌面環境與 thin clinet。</description>
    <description xml:lang="zh_CN">桌面和瘦客户端。</description>
    <description xml:lang="zh_TW">桌面環境與 thin clinet。</description>
    <display_order>70</display_order>
    <grouplist>
      <groupid>fonts</groupid>
      <groupid>gnome-desktop</groupid>
    </grouplist>
  </category>
  <environment>
    <id>custom-environment</id>
    <name>Custom Operating System</name>
    <name xml:lang="de">Benutzerdefiniertes Betriebssystem</name>
    <name xml:lang="es">Sistema operativo personalizado</name>
    <name xml:lang="fr">Custom Operating System</name>
    <name xml:lang="it">Sistema operativo personalizzato</name>
    <name xml:lang="ja">カスタムオペレーティングシステム</name>
    <name xml:lang="ko">사용자 정의 운영 체제</name>
    <name xml:lang="pt">Sistema Operacional Personalizado</name>
    <name xml:lang="pt_BR">Sistema Operacional Personalizado</name>
    <name xml:lang="ru">Пользовательская операционная система</name>
    <name xml:lang="zh_CN">定制操作系统</name>
    <name xml:lang="zh_TW">定制操作系統</name>
    <description>Basic building block for a custom CentOS system.</description>
    <display_order>99</display_order>
    <grouplist>
      <groupid>core</groupid>
    </grouplist>
    <optionlist>
      <groupid>guest-agents</groupid>
      <groupid>standard</groupid>
    </optionlist>
  </environment>
  <environment>
    <id>minimal-environment</id>
    <name>Minimal Install</name>
    <name xml:lang="as">নূন্যতম ইনস্টল</name>
    <name xml:lang="cs">Minimální instalace</name>
    <name xml:lang="de">Minimale Installation</name>
    <name xml:lang="de_CH">Minimale Installation</name>
    <name xml:lang="es">Instalación mínima</name>
    <name xml:lang="fr">Installation minimale</name>
    <name xml:lang="gu">ન્યૂનતમ સ્થાપન</name>
    <name xml:lang="hi">न्यूनतम संस्थापन</name>
    <name xml:lang="it">Installazione minima</name>
    <name xml:lang="ja">最小限のインストール</name>
    <name xml:lang="kn">ಕನಿಷ್ಟ ಅನುಸ್ಥಾಪನೆ</name>
    <name xml:lang="ko">최소 설치</name>
    <name xml:lang="ml">ഏറ്റവും കുറഞ്ഞ ഇന്‍സ്റ്റോള്‍</name>
    <name xml:lang="mr">किमान इंस्टॉल</name>
    <name xml:lang="or">ସର୍ବନିମ୍ନ ସ୍ଥାପନ</name>
    <name xml:lang="pa">ਘੱਟ ਤੋਂ ਘੱਟ ਇੰਸਟਾਲ</name>
    <name xml:lang="pl">Minimalna instalacja</name>
    <name xml:lang="pt">Instalações Mínimas</name>
    <name xml:lang="pt_BR">Instalações Mínimas</name>
    <name xml:lang="ru">Минимальная установка</name>
    <name xml:lang="ta">குறைந்தபட்ச நிறுவல்</name>
    <name xml:lang="te">కనీసపు సంస్థాపన</name>
    <name xml:lang="uk">Мінімальна система</name>
    <name xml:lang="zh">最小型安裝</name>
    <name xml:lang="zh_CN">最小安装</name>
    <name xml:lang="zh_TW">最小型安裝</name>
    <description>Basic functionality.</description>
    <description xml:lang="as">মৌলি কাৰ্য্যকৰীতা।</description>
    <description xml:lang="cs">Základní funkcionalita.</description>
    <description xml:lang="de">Grundlegende Funktionalität.</description>
    <description xml:lang="de_CH">Grundlegende Funktionalität.</description>
    <description xml:lang="es">Funcionalidad básica.</description>
    <description xml:lang="fr">Fonctionnalité de base.</description>
    <description xml:lang="gu">મૂળભૂત વિધેય.</description>
    <description xml:lang="hi">मौलिक प्रकार्यात्मकता.</description>
    <description xml:lang="it">Funzione di base.</description>
    <description xml:lang="ja">基本的な機能です。</description>
    <description xml:lang="kn">ಮೂಲಭೂತ ಕ್ರಿಯಾಶೀಲತೆ.</description>
    <description xml:lang="ko">기본적인 기능입니다.</description>
    <description xml:lang="ml">അടിസ്ഥാന പ്രവൃത്തിവിശേഷണം.</description>
    <description xml:lang="mr">मूळ कार्यक्षमता.</description>
    <description xml:lang="or">ସାଧାରଣ କାର୍ଯ୍ୟକାରିତା।</description>
    <description xml:lang="pa">ਮੁੱਢਲੀ ਕਾਰਜਸ਼ੀਲਤਾ।</description>
    <description xml:lang="pl">Podstawowa funkcjonalność.</description>
    <description xml:lang="pt">Função básica</description>
    <description xml:lang="pt_BR">Função básica</description>
    <description xml:lang="ru">Базовая функциональность.</description>
    <description xml:lang="ta">அடிப்படை செயலம்சம்.</description>
    <description xml:lang="te">ప్రాథమిక ఫంక్షనాలిటి.</description>
    <description xml:lang="uk">Основні можливості.</description>
    <description xml:lang="zh">基本功能。</description>
    <description xml:lang="zh_CN">基本功能。</description>
    <description xml:lang="zh_TW">基本功能。</description>
    <display_order>3</display_order>
    <grouplist>
      <groupid>core</groupid>
    </grouplist>
    <optionlist>
      <groupid>guest-agents</groupid>
      <groupid>standard</groupid>
    </optionlist>
  </environment>
  <environment>
    <id>server-product-environment</id>
    <name>Server</name>
    <name xml:lang="fr">Serveurs</name>
    <name xml:lang="ja">サーバー</name>
    <name xml:lang="ko">서버</name>
    <name xml:lang="zh_CN">服务器</name>
    <description>An integrated, easy-to-manage server.</description>
    <description xml:lang="de">Ein integrierter, einfach zu verwaltender Server.</description>
    <description xml:lang="es">Un servidor integrado y fácil de administrar.</description>
    <description xml:lang="fr">Un serveur intégré, facile à gérer.</description>
    <description xml:lang="it">Un server integrato e facile da gestire.</description>
    <description xml:lang="ja">統合された、管理が容易なサーバーです。</description>
    <description xml:lang="ko">통합되고 관리가 쉬운 서버.</description>
    <description xml:lang="pt">Um servidor integrado e fácil de gerir.</description>
    <description xml:lang="pt_BR">Um servidor integrado e fácil de gerir.</description>
    <description xml:lang="ru">Интегрированный, простой в управлении сервер.</description>
    <description xml:lang="zh_CN">集成的易于管理的服务器。</description>
    <description xml:lang="zh_TW">一個集成，易於管理的服務器。</description>
    <display_order>2</display_order>
    <grouplist>
      <groupid>core</groupid>
      <groupid>hardware-support</groupid>
      <groupid>headless-management</groupid>
      <groupid>server-product</groupid>
      <groupid>standard</groupid>
    </grouplist>
    <optionlist>
      <groupid>debugging</groupid>
      <groupid>dns-server</groupid>
      <groupid>file-server</groupid>
      <groupid>ftp-server</groupid>
      <groupid>gnome-desktop</groupid>
      <groupid>guest-agents</groupid>
      <groupid>hardware-monitoring</groupid>
      <groupid>infiniband</groupid>
      <groupid>mail-server</groupid>
      <groupid>network-file-system-client</groupid>
      <groupid>network-server</groupid>
      <groupid>performance</groupid>
      <groupid>remote-system-management</groupid>
      <groupid>smb-server</groupid>
      <groupid>virtualization-hypervisor</groupid>
      <groupid>web-server</groupid>
    </optionlist>
  </environment>
</comps>"""
BIG_GROUPS = 3
BIG_CATEGORY = 4
BIG_LANGPACK = 0
BIG_ENVIRONMENTS = 3

MODULEMD_FIELDS = [
    "arch",
    "artifacts",
    "context",
    "dependencies",
    "description",
    "name",
    "packages",
    "profiles",
    "pulp_created",
    "pulp_href",
    "static_context",
    "stream",
    "version",
]
